@@ -11,38 +11,25 @@
 
 #include "DirInfo.h"
 #include "DirTree.h"
+#include "DotEntry.h"
+#include "Attic.h"
 #include "FileInfoIterator.h"
 #include "FileInfoSorter.h"
+#include "ExcludeRules.h"
 #include "Exception.h"
+#include "DebugHelpers.h"
+
+#define DIRECT_CHILDREN_COUNT_SANITY_CHECK 0
 
 using namespace QDirStat;
 
 
 DirInfo::DirInfo( DirTree * tree,
-		  DirInfo * parent,
-		  bool	    asDotEntry )
+		  DirInfo * parent )
     : FileInfo( tree, parent )
 {
     init();
-
-    if ( asDotEntry )
-    {
-	_isDotEntry = true;
-	_name	    = dotEntryName();
-
-        if ( parent )
-        {
-            _device = parent->device();
-            _mode   = parent->mode();
-            _uid    = parent->uid();
-            _gid    = parent->gid();
-            _mtime  = parent->mtime();
-        }
-    }
-    else
-    {
-	_readState  = DirFinished;
-    }
+    _readState = DirFinished;
 }
 
 
@@ -56,7 +43,9 @@ DirInfo::DirInfo( const QString & filenameWithoutPath,
 		parent )
 {
     init();
-    _dotEntry = new DirInfo( tree, this, true );
+    ensureDotEntry();
+
+    _directChildrenCount++;
 }
 
 
@@ -74,32 +63,38 @@ DirInfo::DirInfo( DirTree *	  tree,
 		mtime )
 {
     init();
-    _dotEntry = new DirInfo( tree, this, true );
+    ensureDotEntry();
+
+    _directChildrenCount++;
 }
 
 
 void DirInfo::init()
 {
-    _isDotEntry	     = false;
-    _isMountPoint    = false;
-    _isExcluded	     = false;
-    _summaryDirty    = false;
-    _deletingAll     = false;
-    _locked	     = false;
-    _touched	     = false;
-    _pendingReadJobs = 0;
-    _dotEntry	     = 0;
-    _firstChild	     = 0;
-    _totalSize	     = _size;
-    _totalBlocks     = _blocks;
-    _totalItems	     = 0;
-    _totalSubDirs    = 0;
-    _totalFiles	     = 0;
-    _latestMtime     = _mtime;
-    _readState	     = DirQueued;
-    _sortedChildren  = 0;
-    _lastSortCol     = UndefinedCol;
-    _lastSortOrder   = Qt::AscendingOrder;
+    _dotEntry		 = 0;
+    _attic		 = 0;
+    _isMountPoint	 = false;
+    _isExcluded		 = false;
+    _summaryDirty	 = false;
+    _deletingAll	 = false;
+    _locked		 = false;
+    _touched		 = false;
+    _pendingReadJobs	 = 0;
+    _dotEntry		 = 0;
+    _firstChild		 = 0;
+    _totalSize		 = _size;
+    _totalBlocks	 = _blocks;
+    _totalItems		 = 0;
+    _totalSubDirs	 = 0;
+    _totalFiles		 = 0;
+    _totalIgnoredItems	 = 0;
+    _totalUnignoredItems = 0;
+    _directChildrenCount = 0;
+    _latestMtime	 = _mtime;
+    _readState		 = DirQueued;
+    _sortedChildren	 = 0;
+    _lastSortCol	 = UndefinedCol;
+    _lastSortOrder	 = Qt::AscendingOrder;
 }
 
 
@@ -135,6 +130,12 @@ void DirInfo::clear()
 	_dotEntry = 0;
     }
 
+    if ( _attic )
+    {
+	delete _attic;
+	_attic = 0;
+    }
+
     _summaryDirty = true;
     dropSortCache();
 }
@@ -142,47 +143,105 @@ void DirInfo::clear()
 
 void DirInfo::reset()
 {
-    if ( _isDotEntry )
-	return;
-
-    if ( _firstChild || _dotEntry )
+    if ( _firstChild || _dotEntry || _attic )
 	clear();
 
     _readState	     = DirQueued;
     _pendingReadJobs = 0;
     _summaryDirty    = true;
 
-    if ( ! _dotEntry )
-    {
-	_dotEntry = new DirInfo( _tree, this, true );
+    ensureDotEntry();
+
+    if ( _tree )
 	_tree->childAddedNotify( _dotEntry );
-    }
 
     recalc();
     dropSortCache();
 }
 
 
+DotEntry * DirInfo::ensureDotEntry()
+{
+    if ( ! _dotEntry )
+    {
+	// logDebug() << "Creating dot entry for " << this << endl;
+
+	_dotEntry = new DotEntry( _tree, this );
+	CHECK_NEW( _dotEntry );
+    }
+
+    return _dotEntry;
+}
+
+
+void DirInfo::deleteEmptyDotEntry()
+{
+    if ( ! _dotEntry->firstChild() && ! _dotEntry->hasAtticChildren() )
+    {
+	delete _dotEntry;
+	_dotEntry = 0;
+
+	countDirectChildren();
+    }
+}
+
+
+Attic * DirInfo::ensureAttic()
+{
+    if ( ! _attic )
+    {
+	// logDebug() << "Creating attic for " << this << endl;
+
+	_attic = new Attic( _tree, this );
+	CHECK_NEW( _attic );
+    }
+
+    return _attic;
+}
+
+
+void DirInfo::deleteEmptyAttic()
+{
+    if ( _attic && ! _attic->firstChild() )
+    {
+	delete _attic;
+	_attic = 0;
+    }
+}
+
+
+bool DirInfo::hasAtticChildren() const
+{
+    return _attic && _attic->hasChildren();
+}
+
+
 void DirInfo::recalc()
 {
     // logDebug() << this << endl;
 
-    _totalSize	  = _size;
-    _totalBlocks  = _blocks;
-    _totalItems	  = 0;
-    _totalSubDirs = 0;
-    _totalFiles	  = 0;
-    _latestMtime  = _mtime;
+    _totalSize		 = _size;
+    _totalBlocks	 = _blocks;
+    _totalItems		 = 0;
+    _totalSubDirs	 = 0;
+    _totalFiles		 = 0;
+    _totalIgnoredItems	 = 0;
+    _totalUnignoredItems = 0;
+    _directChildrenCount = 0;
+    _latestMtime	 = _mtime;
 
     FileInfoIterator it( this );
 
     while ( *it )
     {
-	_totalSize    += (*it)->totalSize();
-	_totalBlocks  += (*it)->totalBlocks();
-	_totalItems   += (*it)->totalItems() + 1;
-	_totalSubDirs += (*it)->totalSubDirs();
-	_totalFiles   += (*it)->totalFiles();
+	_directChildrenCount++;
+	_totalSize	     += (*it)->totalSize();
+	_totalBlocks	     += (*it)->totalBlocks();
+	_totalItems	     += (*it)->totalItems() + 1;
+	_totalSubDirs	     += (*it)->totalSubDirs();
+	_totalFiles	     += (*it)->totalFiles();
+	_totalIgnoredItems   += (*it)->totalIgnoredItems();
+	_totalUnignoredItems += (*it)->totalUnignoredItems();
 
 	if ( (*it)->isDir() )
 	    _totalSubDirs++;
@@ -190,6 +249,14 @@ void DirInfo::recalc()
 	if ( (*it)->isFile() )
 	    _totalFiles++;
 
+	if ( ! (*it)->isDir() )
+	{
+	    if ( (*it)->isIgnored() )
+		_totalIgnoredItems++;
+	    else
+		_totalUnignoredItems++;
+	}
+
 	time_t childLatestMtime = (*it)->latestMtime();
 
 	if ( childLatestMtime > _latestMtime )
@@ -198,6 +265,9 @@ void DirInfo::recalc()
 	++it;
     }
 
+    if ( _attic )
+	_totalIgnoredItems += _attic->totalIgnoredItems();
+
     _summaryDirty = false;
 }
 
@@ -253,6 +323,33 @@ int DirInfo::totalFiles()
 }
 
 
+int DirInfo::totalNonDirItems()
+{
+    if ( _summaryDirty )
+	recalc();
+
+    return _totalItems - _totalSubDirs;
+}
+
+
+int DirInfo::totalIgnoredItems()
+{
+    if ( _summaryDirty )
+	recalc();
+
+    return _totalIgnoredItems;
+}
+
+
+int DirInfo::totalUnignoredItems()
+{
+    if ( _summaryDirty )
+	recalc();
+
+    return _totalUnignoredItems;
+}
+
+
 time_t DirInfo::latestMtime()
 {
     if ( _summaryDirty )
@@ -262,6 +359,36 @@ time_t DirInfo::latestMtime()
 }
 
 
+int DirInfo::directChildrenCount()
+{
+    if ( _summaryDirty )
+	recalc();
+
+    return _directChildrenCount;
+}
+
+
+int DirInfo::countDirectChildren()
+{
+    // logDebug() << this << endl;
+
+    _directChildrenCount = 0;
+
+    FileInfo * child = _firstChild;
+
+    while ( child )
+    {
+	++_directChildrenCount;
+	child = child->next();
+    }
+
+    if ( _dotEntry )
+	++_directChildrenCount;
+
+    return _directChildrenCount;
+}
+
+
 bool DirInfo::isFinished()
 {
     return ! isBusy();
@@ -292,20 +419,11 @@ bool DirInfo::isBusy()
 }
 
 
-void DirInfo::setDotEntry( FileInfo *newDotEntry )
-{
-    if ( newDotEntry )
-	_dotEntry = newDotEntry->toDirInfo();
-    else
-	_dotEntry = 0;
-}
-
-
-void DirInfo::insertChild( FileInfo *newChild )
+void DirInfo::insertChild( FileInfo * newChild )
 {
     CHECK_PTR( newChild );
 
-    if ( newChild->isDir() || _dotEntry == 0 || _isDotEntry )
+    if ( newChild->isDir() || ! _dotEntry )
     {
 	/**
 	 * Only directories are stored directly in pure directory nodes -
@@ -339,34 +457,92 @@ void DirInfo::insertChild( FileInfo *newChild )
 }
 
 
-void DirInfo::childAdded( FileInfo *newChild )
+void DirInfo::moveToAttic( FileInfo * child )
 {
-    if ( ! _summaryDirty )
+    unlinkChild( child );
+    addToAttic( child );
+}
+
+
+void DirInfo::addToAttic( FileInfo * newChild )
+{
+    CHECK_PTR( newChild );
+
+    Attic * attic = 0;
+
+    if ( ! newChild->isDir() && _dotEntry )
+	attic = _dotEntry->ensureAttic();
+
+    if ( ! attic )
+	attic = ensureAttic();
+
+    newChild->setIgnored( true );
+
+    if ( newChild->isDir() )
+        _totalIgnoredItems += newChild->totalIgnoredItems();
+    else
+        _totalIgnoredItems++;
+
+    CHECK_PTR( attic );
+    attic->insertChild( newChild );
+}
+
+
+void DirInfo::childAdded( FileInfo * newChild )
+{
+    bool addToTotal = true;
+
+    if ( newChild->isIgnored() )
     {
-	_totalSize   += newChild->size();
-	_totalBlocks += newChild->blocks();
-	_totalItems++;
+        if ( newChild->isDir() )
+            _totalIgnoredItems += newChild->totalIgnoredItems();
+        else
+            _totalIgnoredItems++;
 
-	if ( newChild->isDir() )
-	    _totalSubDirs++;
+	// Add ignored items only to all the totals if this directory is also
+	// ignored or if this is the attic.
 
-	if ( newChild->isFile() )
-	    _totalFiles++;
-
-	if ( newChild->mtime() > _latestMtime )
-	    _latestMtime = newChild->mtime();
+	if ( ! _isIgnored &&  ! isAttic() )
+	    addToTotal = false;
     }
     else
     {
-	// NOP
+	if ( ! newChild->isDir() )
+	    _totalUnignoredItems++;
+    }
 
-	/*
-	 * Don't bother updating the summary fields if the summary is dirty
-	 * (i.e. outdated) anyway: As soon as anybody wants to know some exact
-	 * value a complete recalculation of the entire subtree will be
-	 * triggered. On the other hand, if nobody wants to know (which is very
-	 * likely) we can save this effort.
-	 */
+    if ( addToTotal )
+    {
+	if ( ! _summaryDirty )
+	{
+	    _totalSize	 += newChild->size();
+	    _totalBlocks += newChild->blocks();
+	    _totalItems++;
+
+	    if ( newChild->parent() == this )
+		_directChildrenCount++;
+
+	    if ( newChild->isDir() )
+		_totalSubDirs++;
+
+	    if ( newChild->isFile() )
+		_totalFiles++;
+
+	    if ( newChild->mtime() > _latestMtime )
+		_latestMtime = newChild->mtime();
+	}
+	else
+	{
+	    // NOP
+
+	    /*
+	     * Don't bother updating the summary fields if the summary is dirty
+	     * (i.e. outdated) anyway: As soon as anybody wants to know some
+	     * exact value a complete recalculation of the entire subtree will
+	     * be triggered. On the other hand, if nobody wants to know (which
+	     * is very likely) we can save this effort.
+	     */
+	}
     }
 
     if ( _lastSortCol != ReadJobsCol )
@@ -388,7 +564,7 @@ void DirInfo::deletingChild( FileInfo * child )
      * recalculated: The child now being deleted might just be the one with the
      * latest mtime, and figuring out the second-latest cannot easily be
      * done. So we merely mark the summary as dirty and wait until a recalc()
-     * will be triggered from outside - which might as well never happen when
+     * will be triggered from outside - which might as well never happen if
      * nobody wants to know some summary field anyway.
      **/
 
@@ -418,7 +594,7 @@ void DirInfo::deletingChild( FileInfo * child )
 }
 
 
-void DirInfo::unlinkChild( FileInfo *deletedChild )
+void DirInfo::unlinkChild( FileInfo * deletedChild )
 {
     if ( deletedChild->parent() != this )
     {
@@ -428,21 +604,22 @@ void DirInfo::unlinkChild( FileInfo *deletedChild )
     }
 
     dropSortCache();
+    _summaryDirty = true;
 
     if ( deletedChild == _firstChild )
     {
-	logDebug() << "Unlinking first child " << deletedChild << endl;
+	// logDebug() << "Unlinking first child " << deletedChild << endl;
 	_firstChild = deletedChild->next();
 	return;
     }
 
-    FileInfo *child = firstChild();
+    FileInfo * child = firstChild();
 
     while ( child )
     {
 	if ( child->next() == deletedChild )
 	{
-	    logDebug() << "Unlinking " << deletedChild << endl;
+	    // logDebug() << "Unlinking " << deletedChild << endl;
 	    child->setNext( deletedChild->next() );
 
 	    return;
@@ -489,19 +666,25 @@ void DirInfo::readJobAborted()
 }
 
 
+DirReadState DirInfo::readState() const
+{
+    return _readState;
+}
+
+
 void DirInfo::finalizeLocal()
 {
     // logDebug() << this << endl;
+
     cleanupDotEntries();
+    cleanupAttics();
+    checkIgnored();
 }
 
 
 void DirInfo::finalizeAll()
 {
-    if ( _isDotEntry )
-	return;
-
-    FileInfo *child = firstChild();
+    FileInfo * child = firstChild();
 
     while ( child )
     {
@@ -522,57 +705,102 @@ void DirInfo::finalizeAll()
     // get all their plain file children reparented to themselves, so they
     // would need to be processed in the loop, too.
 
-     _tree->sendFinalizeLocal( this ); // Must be sent _before_ finalizeLocal()!
     finalizeLocal();
 }
 
 
-DirReadState DirInfo::readState() const
-{
-    if ( _isDotEntry && _parent )
-	return _parent->readState();
-    else
-	return _readState;
-}
-
-
 void DirInfo::cleanupDotEntries()
 {
-    if ( ! _dotEntry || _isDotEntry )
+    if ( ! _dotEntry )
 	return;
 
     // Reparent dot entry children if there are no subdirectories on this level
 
-    if ( ! _firstChild )
+    if ( ! _firstChild && ! hasAtticChildren() )
     {
-	FileInfo *child = _dotEntry->firstChild();
+	takeAllChildren( _dotEntry );
 
-	if ( child )
+	// Reparent the dot entry's attic's children to this item's attic
+
+	if ( _dotEntry->hasAtticChildren() )
 	{
-	    // logDebug() << "Reparenting children of solo dot entry " << this << endl;
-	    _firstChild = child;	    // Move the entire children chain here.
-	    _dotEntry->setFirstChild( 0 );  // _dotEntry will be deleted below.
-
-	    while ( child )
-	    {
-		child->setParent( this );
-		child = child->next();
-	    }
+	    ensureAttic();
+	    _attic->takeAllChildren( _dotEntry->attic() );
 	}
     }
 
-
-    // Delete dot entries without any children.
+    // Delete the dot entry if it is now empty.
     //
     // This also affects dot entries that were just disowned because they had
     // no siblings (i.e., there are no subdirectories on this level).
+    //
+    // Notice that this also checks if the dot entry's attic (if it has one) is
+    // empty, and that its attic is deleted along with the dot entry.
 
-    if ( ! _dotEntry->firstChild() )
+    deleteEmptyDotEntry();
+}
+
+
+void DirInfo::cleanupAttics()
+{
+    if ( _dotEntry )
+	_dotEntry->cleanupAttics();
+
+    if ( _attic )
     {
-	// logDebug() << "Removing empty dot entry " << this << endl;
+	_attic->finalizeLocal();
 
-	delete _dotEntry;
-	_dotEntry = 0;
+	if ( ! _attic->firstChild() && ! _attic->dotEntry() )
+	{
+	    delete _attic;
+	    _attic = 0;
+
+	    if ( _lastIncludeAttic )
+		dropSortCache();
+
+	    _summaryDirty = true;
+	}
+    }
+}
+
+
+void DirInfo::checkIgnored()
+{
+    if ( _dotEntry )
+	_dotEntry->checkIgnored();
+
+    // Cascade the 'ignored' status up the tree:
+    //
+    // Display all directories as ignored that have any ignored items, but no
+    // items that are not ignored.
+
+    _isIgnored = ( totalIgnoredItems() > 0 && totalUnignoredItems() == 0 );
+
+    if ( _isIgnored )
+	ignoreEmptySubDirs();
+
+    if ( ! isPseudoDir() && _parent )
+	_parent->checkIgnored();
+}
+
+
+void DirInfo::ignoreEmptySubDirs()
+{
+    FileInfoIterator it( this );
+
+    while ( *it )
+    {
+	if ( ! (*it)->isIgnored() && (*it)->isDirInfo() )
+	{
+	    if ( (*it)->totalUnignoredItems() == 0 )
+	    {
+		// logDebug() << "Ignoring empty subdir " << (*it) << endl;
+		(*it)->setIgnored( true );
+                _summaryDirty = true;
+	    }
+	}
+
+	++it;
     }
 }
 
@@ -581,29 +809,41 @@ void DirInfo::clearTouched( bool recursive )
 {
     _touched = false;
 
-    if ( recursive && ! _isDotEntry )
+    if ( recursive )
     {
-	FileInfo * child = _firstChild;
+        if ( ! isDotEntry() )
+        {
+            FileInfo * child = _firstChild;
 
-	while ( child )
-	{
-	    if ( child->isDirInfo() )
-		child->toDirInfo()->clearTouched();
+            while ( child )
+            {
+                if ( child->isDirInfo() )
+                    child->toDirInfo()->clearTouched();
 
-	    child = child->next();
-	}
+                child = child->next();
+            }
 
-	if ( _dotEntry )
-	    _dotEntry->clearTouched();
+            if ( _dotEntry )
+                _dotEntry->clearTouched();
+        }
+
+        if ( _attic )
+            _attic->clearTouched();
     }
 }
 
 
 const FileInfoList & DirInfo::sortedChildren( DataColumn    sortCol,
-					      Qt::SortOrder sortOrder )
+					      Qt::SortOrder sortOrder,
+					      bool	    includeAttic )
 {
-    if ( _sortedChildren && sortCol == _lastSortCol && sortOrder == _lastSortOrder )
+    if ( _sortedChildren &&
+	 sortCol      == _lastSortCol	   &&
+	 sortOrder    == _lastSortOrder	   &&
+	 includeAttic == _lastIncludeAttic    )
+    {
 	return *_sortedChildren;
+    }
 
 
     // Clean old sorted children list and create a new one
@@ -631,12 +871,42 @@ const FileInfoList & DirInfo::sortedChildren( DataColumn    sortCol,
 
     // logDebug() << "Sorting children of " << this << " by " << sortCol << endl;
 
+    if ( sortCol != NameCol )
+    {
+	// Do secondary sorting by NameCol (always in ascending order)
+
+	std::stable_sort( _sortedChildren->begin(),
+			  _sortedChildren->end(),
+			  FileInfoSorter( NameCol, Qt::AscendingOrder ) );
+    }
+
+
+    // Primary sorting by sortCol ascending or descending (as specified in sortOrder)
+
     std::stable_sort( _sortedChildren->begin(),
 		      _sortedChildren->end(),
 		      FileInfoSorter( sortCol, sortOrder ) );
 
-    _lastSortCol   = sortCol;
-    _lastSortOrder = sortOrder;
+    if ( includeAttic && _attic )
+	_sortedChildren->append( _attic );
+
+    _lastSortCol      = sortCol;
+    _lastSortOrder    = sortOrder;
+    _lastIncludeAttic = includeAttic;
+
+
+#if DIRECT_CHILDREN_COUNT_SANITY_CHECK
+
+    if ( _sortedChildren->size() != _directChildrenCount )
+    {
+	Debug::dumpChildrenList( this, *_sortedChildren );
+
+	THROW( Exception( QString( "_directChildrenCount of %1 corrupted; is %2, should be %3" )
+			  .arg( debugUrl() )
+			  .arg( _directChildrenCount )
+			  .arg( _sortedChildren->size() ) ) );
+    }
+#endif
 
     return *_sortedChildren;
 }
@@ -665,21 +935,27 @@ void DirInfo::dropSortCache( bool recursive )
 	// any in the subtree, either. And dot entries don't have dir children
 	// that could have a sort cache.
 
-	if ( recursive && ! _isDotEntry )
-	{
-	    FileInfo * child = _firstChild;
+	if ( recursive )
+        {
+            if ( ! isDotEntry() )
+            {
+                FileInfo * child = _firstChild;
 
-	    while ( child )
-	    {
-		if ( child->isDirInfo() )
-		    child->toDirInfo()->dropSortCache( recursive );
+                while ( child )
+                {
+                    if ( child->isDirInfo() )
+                        child->toDirInfo()->dropSortCache( recursive );
 
-		child = child->next();
-	    }
+                    child = child->next();
+                }
 
-	    if ( _dotEntry )
-		_dotEntry->dropSortCache( recursive );
-	}
+                if ( _dotEntry )
+                    _dotEntry->dropSortCache( recursive );
+            }
+
+            if ( _attic )
+                _attic->dropSortCache( recursive );
+        }
     }
 }
 
@@ -689,7 +965,37 @@ const DirInfo * DirInfo::findNearestMountPoint() const
     const DirInfo * dir = this;
 
     while ( dir && ! dir->isMountPoint() )
-        dir = dir->parent();
+	dir = dir->parent();
 
     return dir;
 }
+
+
+void DirInfo::takeAllChildren( DirInfo * oldParent )
+{
+    FileInfo * child = oldParent->firstChild();
+
+    if ( child )
+    {
+	// logDebug() << "Reparenting all children of " << oldParent << " to " << this << endl;
+
+	FileInfo * oldFirstChild = _firstChild;
+	_firstChild = child;
+	FileInfo * lastChild = child;
+
+	oldParent->setFirstChild( 0 );
+	oldParent->recalc();
+
+	_directChildrenCount = -1;
+	_summaryDirty	     = true;
+
+	while ( child )
+	{
+	    child->setParent( this );
+	    lastChild = child;
+	    child = child->next();
+	}
+
+	lastChild->setNext( oldFirstChild );
+    }
+}
