@@ -11,7 +11,6 @@
 #define DirInfo_h
 
 
-#include "Logger.h"
 #include "FileInfo.h"
 #include "DataColumns.h"
 
@@ -20,29 +19,19 @@ namespace QDirStat
 {
     // Forward declarations
     class DirTree;
+    class DotEntry;
 
     /**
-     * A more specialized version of @ref FileInfo: This class can actually
-     * manage children. The base class (@ref FileInfo) has only stubs for the
-     * respective methods to integrate seamlessly with the abstraction of a
-     * file / directory tree; this class fills those stubs with life.
+     * A more specialized version of FileInfo: This class can actually manage
+     * children. The base class (FileInfo) has only stubs for the respective
+     * methods to integrate seamlessly with the abstraction of a file /
+     * directory tree; this class fills those stubs with life.
      *
-     * @short directory item within a @ref DirTree.
+     * @short directory item within a DirTree.
      **/
     class DirInfo: public FileInfo
     {
     public:
-	/**
-	 * Default constructor.
-	 *
-	 * If "asDotEntry" is set, this will be used as the parent's
-	 * "dot entry", i.e. the pseudo directory that holds all the parent's
-	 * non-directory children. This is the only way to create a "dot
-	 * entry"!
-	 **/
-	DirInfo( DirTree * tree,
-		 DirInfo * parent     = 0,
-		 bool	   asDotEntry = false );
 
 	/**
 	 * Constructor from a stat buffer (i.e. based on an lstat() call).
@@ -53,8 +42,8 @@ namespace QDirStat
 		 DirInfo       * parent = 0 );
 
 	/**
-	 * Constructor from the bare neccessary fields
-	 * for use from a cache file reader
+	 * Constructor from the bare necessary fields for use from a cache file
+	 * reader.
 	 **/
 	DirInfo( DirTree *	 tree,
 		 DirInfo *	 parent,
@@ -63,6 +52,16 @@ namespace QDirStat
 		 FileSize	 size,
 		 time_t		 mtime );
 
+	/**
+	 * Default constructor.
+	 *
+	 * This is the only constructor that will not create a dot entry
+	 * immediately. If that is desired, you can always use ensureDotEntry()
+	 * later.
+	 **/
+	DirInfo( DirTree * tree,
+		 DirInfo * parent = 0 );
+
 	/**
 	 * Destructor.
 	 **/
@@ -71,14 +70,14 @@ namespace QDirStat
 	/**
 	 * Returns the total size in bytes of this subtree.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual FileSize totalSize() Q_DECL_OVERRIDE;
 
 	/**
 	 * Returns the total size in blocks of this subtree.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual FileSize totalBlocks() Q_DECL_OVERRIDE;
 
@@ -86,7 +85,7 @@ namespace QDirStat
 	 * Returns the total number of children in this subtree, excluding this
 	 * item.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual int totalItems() Q_DECL_OVERRIDE;
 
@@ -94,7 +93,7 @@ namespace QDirStat
 	 * Returns the total number of subdirectories in this subtree,
 	 * excluding this item. Dot entries and "." or ".." are not counted.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual int totalSubDirs() Q_DECL_OVERRIDE;
 
@@ -102,14 +101,52 @@ namespace QDirStat
 	 * Returns the total number of plain file children in this subtree,
 	 * excluding this item.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual int totalFiles() Q_DECL_OVERRIDE;
 
+	/**
+	 * Returns the total number of non-directory items in this subtree,
+	 * excluding this item.
+	 *
+	 * Reimplemented - inherited from FileInfo.
+	 **/
+	virtual int totalNonDirItems() Q_DECL_OVERRIDE;
+
+	/**
+	 * Returns the total number of ignored (non-directory!) items in this
+	 * subtree, excluding this item.
+	 *
+	 * Reimplemented - inherited from FileInfo.
+	 **/
+	virtual int totalIgnoredItems() Q_DECL_OVERRIDE;
+
+	/**
+	 * Returns the total number of not ignored (non-directory!) items in
+	 * this subtree, excluding this item.
+	 *
+	 * Reimplemented - inherited from FileInfo.
+	 **/
+	virtual int totalUnignoredItems() Q_DECL_OVERRIDE;
+
+	/**
+	 * Returns the total number of direct children of this directory.
+	 *
+	 * If this directory has a dot entry, the dot entry itself is counted,
+	 * but not the file children of the dot entry.
+	 *
+	 * This method uses a cached value whenever possible, so it is
+	 * considerably faster than the unconditional countDirectChildren()
+	 * method.
+	 *
+	 * Reimplemented - inherited from FileInfo.
+	 **/
+	virtual int directChildrenCount() Q_DECL_OVERRIDE;
+
 	/**
 	 * Returns the latest modification time of this subtree.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual time_t latestMtime() Q_DECL_OVERRIDE;
 
@@ -131,7 +168,7 @@ namespace QDirStat
 	 * This will return 'false' only if this information can be obtained at
 	 * all, i.e. if local directory reading methods are used.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual bool isMountPoint() const  Q_DECL_OVERRIDE
 	    { return _isMountPoint; }
@@ -140,20 +177,20 @@ namespace QDirStat
 	 * Sets the mount point state, i.e. whether or not this is a mount
 	 * point.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual void setMountPoint( bool isMountPoint = true ) Q_DECL_OVERRIDE;
 
-        /**
-         * Find the nearest parent that is a mount point or 0 if there is
-         * none. This may return this DirInfo itself.
-         **/
-        const DirInfo * findNearestMountPoint() const;
+	/**
+	 * Find the nearest parent that is a mount point or 0 if there is
+	 * none. This may return this DirInfo itself.
+	 **/
+	const DirInfo * findNearestMountPoint() const;
 
 	/**
 	 * Returns true if this subtree is finished reading.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual bool isFinished() Q_DECL_OVERRIDE;
 
@@ -161,7 +198,7 @@ namespace QDirStat
 	 * Returns true if this subtree is busy, i.e. it is not finished
 	 * reading yet.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual bool isBusy() Q_DECL_OVERRIDE;
 
@@ -169,7 +206,7 @@ namespace QDirStat
 	 * Returns the number of pending read jobs in this subtree. When this
 	 * number reaches zero, the entire subtree is done.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual int pendingReadJobs() Q_DECL_OVERRIDE
 	    { return _pendingReadJobs;	}
@@ -185,9 +222,9 @@ namespace QDirStat
 	 * Set this entry's first child.
 	 * Use this method only if you know exactly what you are doing.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
-	virtual void setFirstChild( FileInfo *newfirstChild ) Q_DECL_OVERRIDE
+	virtual void setFirstChild( FileInfo * newfirstChild ) Q_DECL_OVERRIDE
 	    { _firstChild = newfirstChild; }
 
 	/**
@@ -196,7 +233,22 @@ namespace QDirStat
 	 * The order of children in this list is absolutely undefined;
 	 * don't rely on any implementation-specific order.
 	 **/
-	virtual void insertChild( FileInfo *newChild ) Q_DECL_OVERRIDE;
+	virtual void insertChild( FileInfo * newChild ) Q_DECL_OVERRIDE;
+
+	/**
+	 * Add a child to the attic. This is very much like insertChild(), but
+	 * it inserts the child into the appropriate attic instead (and sets
+	 * its 'ignored' flag: The dot entry's attic if there is a dot entry
+	 * and the new child is not a directory, and the directory's attic
+	 * otherwise.
+	 **/
+	virtual void addToAttic( FileInfo * newChild );
+
+	/**
+	 * Move a child to the attic, i.e. remove it from the normal children
+	 * and move it to the attic instead.
+	 **/
+	virtual void moveToAttic( FileInfo * newChild );
 
 	/**
 	 * Get the "Dot Entry" for this node if there is one (or 0 otherwise):
@@ -205,29 +257,55 @@ namespace QDirStat
 	 * user can easily tell which summary fields belong to the directory
 	 * itself and which are the accumulated values of the entire subtree.
 	 **/
-	virtual DirInfo * dotEntry() const Q_DECL_OVERRIDE
+	virtual DotEntry * dotEntry() const Q_DECL_OVERRIDE
 	    { return _dotEntry; }
 
 	/**
-	 * Set a "Dot Entry". This makes sense for directories only.
+	 * Return the dot entry for this node. If it doesn't have one yet,
+	 * create it first.
 	 **/
-	virtual void setDotEntry( FileInfo *newDotEntry ) Q_DECL_OVERRIDE;
+	virtual DotEntry * ensureDotEntry();
 
 	/**
-	 * Returns true if this is a "Dot Entry". See @ref dotEntry() for
-	 * details.
-	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Delete the dot entry if it is empty, i.e. it does not have any
+	 * children or its attic (if it has one) is also empty. The dot entry's
+	 * attic is implicitly deleted along with it.
 	 **/
-	virtual bool isDotEntry() const Q_DECL_OVERRIDE
-	    { return _isDotEntry; }
+	virtual void deleteEmptyDotEntry();
+
+	/**
+	 * Return the "Attic" entry for this node if there is one (or 0
+	 * otherwise): This is a pseudo entry that directory nodes use to store
+	 * ignored files and directories separately from the normal tree
+	 * hierarchy.
+	 *
+	 * Reimplemented - inherited from FileInfo.
+	 **/
+	virtual Attic * attic() const Q_DECL_OVERRIDE
+	    { return _attic; }
+
+	/**
+	 * Return the attic for this node. If it doesn't have one yet, create
+	 * it first.
+	 **/
+	virtual Attic * ensureAttic();
+
+	/**
+	 * Delete the attic if it is empty.
+	 **/
+	virtual void deleteEmptyAttic();
+
+	/**
+	 * Return 'true' if there is an attic and it has any children.
+	 **/
+	bool hasAtticChildren() const;
 
 	/**
 	 * Notification that a child has been added somewhere in the subtree.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
-	virtual void childAdded( FileInfo *newChild ) Q_DECL_OVERRIDE;
+	virtual void childAdded( FileInfo * newChild ) Q_DECL_OVERRIDE;
 
 	/**
 	 * Remove a child from the children list.
@@ -237,17 +315,17 @@ namespace QDirStat
 	 * destructor: Important parts of the object might already be destroyed
 	 * (e.g., the virtual table - no more virtual methods).
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
-	virtual void unlinkChild( FileInfo *deletedChild ) Q_DECL_OVERRIDE;
+	virtual void unlinkChild( FileInfo * deletedChild ) Q_DECL_OVERRIDE;
 
 	/**
 	 * Notification that a child is about to be deleted somewhere in the
 	 * subtree.
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
-	virtual void deletingChild( FileInfo *deletedChild ) Q_DECL_OVERRIDE;
+	virtual void deletingChild( FileInfo * deletedChild ) Q_DECL_OVERRIDE;
 
 	/**
 	 * Notification of a new directory read job somewhere in the subtree.
@@ -267,9 +345,9 @@ namespace QDirStat
 	void readJobAborted();
 
 	/**
-	 * Finalize this directory level after reading it is completed.
-	 * This does _not_ mean reading reading all subdirectories is completed
-	 * as well!
+	 * Finalize this directory level after reading it is completed. This
+	 * does _not_ mean that reading all subdirectories is completed as
+	 * well!
 	 *
 	 * Clean up unneeded dot entries.
 	 **/
@@ -279,7 +357,7 @@ namespace QDirStat
 	 * Recursively finalize all directories from here on -
 	 * call finalizeLocal() recursively.
 	 **/
-	void finalizeAll();
+	virtual void finalizeAll();
 
 	/**
 	 * Get the current state of the directory reading process:
@@ -290,26 +368,29 @@ namespace QDirStat
 	 *    DirAborted   reading aborted upon user request
 	 *    DirError	   error while reading
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual DirReadState readState() const Q_DECL_OVERRIDE;
 
 	/**
 	 * Set the state of the directory reading process.
-	 * See @ref readState() for details.
+	 * See readState() for details.
 	 **/
 	void setReadState( DirReadState newReadState );
 
 	/**
 	 * Return a list of (direct) children sorted by 'sortCol' and
-	 * 'sortOrder' (Qt::AscendingOrder or Qt::DescendingOrder).
+	 * 'sortOrder' (Qt::AscendingOrder or Qt::DescendingOrder).  If
+	 * 'includeAttic' is 'true', the attic (if there is one) is added to
+	 * the list.
 	 *
-	 * This might return cached information if the sort column and order
-	 * are the same as for the last call to this function, and there were
-	 * no children added or removed in the meantime.
+	 * This might return cached information if all parameters are the same
+	 * as for the last call to this function, and there were no children
+	 * added or removed in the meantime.
 	 **/
 	const FileInfoList & sortedChildren( DataColumn	   sortCol,
-					     Qt::SortOrder sortOrder );
+					     Qt::SortOrder sortOrder,
+					     bool	   includeAttic = false );
 
 	/**
 	 * Drop all cached information about children sorting.
@@ -346,7 +427,7 @@ namespace QDirStat
 	 * children if there are any, restore the dot entry if it was removed
 	 * (e.g. in finalizeLocal()), set the read state to DirQueued.
 	 **/
-	void reset();
+	virtual void reset();
 
 	/**
 	 * Mark this directory as 'touched'. Item models can use this to keep
@@ -368,19 +449,42 @@ namespace QDirStat
 	void clearTouched( bool recursive = false );
 
 	/**
-	 * Returns true if this is a @ref DirInfo object.
+	 * Returns true if this is a DirInfo object.
 	 *
-	 * Don't confuse this with @ref isDir() which tells whether or not this
-	 * is a disk directory! Both should return the same, but you'll never
-	 * know - better be safe than sorry!
+	 * Don't confuse this with isDir() which tells whether or not this is a
+	 * disk directory! Both should return the same, but you'll never know -
+	 * better be safe than sorry!
 	 *
-	 * Reimplemented - inherited from @ref FileInfo.
+	 * Notice that DotEntry inherits DirInfo, so a DotEntry is also
+	 * implicitly a DirInfo.
+	 *
+	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual bool isDirInfo() const Q_DECL_OVERRIDE
 	    { return true; }
 
+	/**
+	 * Count the direct children unconditionally and update
+	 * _directChildrenCount.
+	 **/
+	int countDirectChildren();
 
-    protected:
+	/**
+	 * Check the 'ignored' state of this item and set the '_isIgnored' flag
+	 * accordingly.
+	 **/
+	virtual void checkIgnored();
+
+	/**
+	 * Set any empty subdir children to ignored. This affects only direct
+	 * children.
+	 **/
+	virtual void ignoreEmptySubDirs();
+
+	/**
+	 * Take all children from 'oldParent' and move them to this DirInfo.
+	 **/
+	virtual void takeAllChildren( DirInfo * oldParent );
 
 	/**
 	 * Recursively recalculate the summary fields when they are dirty.
@@ -390,20 +494,28 @@ namespace QDirStat
 	 **/
 	void recalc();
 
+
+    protected:
+
 	/**
 	 * Clean up unneeded / undesired dot entries:
 	 * Delete dot entries that don't have any children,
 	 * reparent dot entry children to the "real" (parent) directory if
 	 * there are not subdirectory siblings at the level of the dot entry.
 	 **/
-	void cleanupDotEntries();
+	virtual void cleanupDotEntries();
+
+	/**
+	 * Clean up unneeded attics: Delete attic entries that don't have any
+	 * children.
+	 **/
+	virtual void cleanupAttics();
 
 
 	//
 	// Data members
 	//
 
-	bool		_isDotEntry:1;		// Flag: is this entry a "dot entry"?
 	bool		_isMountPoint:1;	// Flag: is this a mount point?
 	bool		_isExcluded:1;		// Flag: was this directory excluded?
 	bool		_summaryDirty:1;	// dirty flag for the cached values
@@ -415,7 +527,8 @@ namespace QDirStat
 	// Children management
 
 	FileInfo *	_firstChild;		// pointer to the first child
-	DirInfo	 *	_dotEntry;		// pseudo entry to hold non-dir children
+	DotEntry *	_dotEntry;		// pseudo entry to hold non-dir children
+	Attic	 *	_attic;			// pseudo entry to hold ignored children
 
 	// Some cached values
 
@@ -424,11 +537,15 @@ namespace QDirStat
 	int		_totalItems;
 	int		_totalSubDirs;
 	int		_totalFiles;
+	int		_totalIgnoredItems;
+	int		_totalUnignoredItems;
+	int		_directChildrenCount;
 	time_t		_latestMtime;
 
 	FileInfoList *	_sortedChildren;
 	DataColumn	_lastSortCol;
 	Qt::SortOrder	_lastSortOrder;
+	bool		_lastIncludeAttic;
 
 	DirReadState	_readState;
 
