@@ -7,6 +7,10 @@
  */
 
 
+#include <QPalette>
+#include <QGuiApplication>
+
+#include "Qt4Compat.h"
 #include "DirTreeModel.h"
 #include "DirTree.h"
 #include "FileInfoIterator.h"
@@ -29,7 +33,8 @@ DirTreeModel::DirTreeModel( QObject * parent ):
     _slowUpdateMillisec( 3000 ),
     _slowUpdate( false ),
     _sortCol( NameCol ),
-    _sortOrder( Qt::AscendingOrder )
+    _sortOrder( Qt::AscendingOrder ),
+    _removingRows( false )
 {
     createTree();
     readSettings();
@@ -138,7 +143,7 @@ void DirTreeModel::clear()
 }
 
 
-void DirTreeModel::openUrl( const QString &url )
+void DirTreeModel::openUrl( const QString & url )
 {
     CHECK_PTR( _tree );
 
@@ -150,6 +155,19 @@ void DirTreeModel::openUrl( const QString &url )
 }
 
 
+void DirTreeModel::readPkg( const PkgFilter & pkgFilter )
+{
+    // logDebug() << "Reading " << pkgFilter << endl;
+    CHECK_PTR( _tree );
+
+    if ( _tree->root() &&  _tree->root()->hasChildren() )
+	clear();
+
+    _updateTimer.start();
+    _tree->readPkg( pkgFilter );
+}
+
+
 void DirTreeModel::loadIcons()
 {
     if ( _treeIconDir.isEmpty() )
@@ -170,8 +188,12 @@ void DirTreeModel::loadIcons()
     _stopIcon	       = QPixmap( _treeIconDir + "stop.png"	      );
     _excludedIcon      = QPixmap( _treeIconDir + "excluded.png"	      );
     _blockDeviceIcon   = QPixmap( _treeIconDir + "block-device.png"   );
-    _charDeviceIcon    = QPixmap( _treeIconDir + "char-device.png"   );
-    _specialIcon       = QPixmap( _treeIconDir + "special.png"	 );
+    _charDeviceIcon    = QPixmap( _treeIconDir + "char-device.png"    );
+    _specialIcon       = QPixmap( _treeIconDir + "special.png"	      );
+    _pkgIcon	       = QPixmap( _treeIconDir + "folder-pkg.png"     );
+#if 0
+    _pkgIcon	       = QPixmap( _treeIconDir + "package.png"	      );
+#endif
 }
 
 
@@ -188,7 +210,8 @@ FileInfo * DirTreeModel::findChild( DirInfo * parent, int childNo ) const
     CHECK_PTR( parent );
 
     const FileInfoList & childrenList =
-	parent->sortedChildren( _sortCol, _sortOrder );
+	parent->sortedChildren( _sortCol, _sortOrder,
+				true );	    // includeAttic
 
     if ( childNo < 0 || childNo >= childrenList.size() )
     {
@@ -212,7 +235,8 @@ int DirTreeModel::rowNumber( FileInfo * child ) const
 	return 0;
 
     const FileInfoList & childrenList =
-	child->parent()->sortedChildren( _sortCol, _sortOrder );
+	child->parent()->sortedChildren( _sortCol, _sortOrder,
+					 true ); // includeAttic
 
     int row = childrenList.indexOf( child );
 
@@ -231,20 +255,12 @@ int DirTreeModel::rowNumber( FileInfo * child ) const
 }
 
 
-int DirTreeModel::countDirectChildren( FileInfo * parent ) const
-{
-    FileInfoIterator it( parent );
-
-    return it.count();
-}
-
-
 
 //
 // Reimplented from QAbstractItemModel
 //
 
-int DirTreeModel::rowCount( const QModelIndex &parentIndex ) const
+int DirTreeModel::rowCount( const QModelIndex & parentIndex ) const
 {
     if ( ! _tree )
 	return 0;
@@ -302,14 +318,14 @@ int DirTreeModel::rowCount( const QModelIndex &parentIndex ) const
 	    if ( _tree->isBusy() )
 		count = 0;
 	    else
-		count = countDirectChildren( item );
+		count = directChildrenCount( item );
 	    break;
 
 	case DirFinished:
 	case DirOnRequestOnly:
 	case DirCached:
 	case DirAborted:
-	    count = countDirectChildren( item );
+	    count = directChildrenCount( item );
 	    break;
 
 	// intentionally omitting 'default' case so the compiler can report
@@ -321,7 +337,7 @@ int DirTreeModel::rowCount( const QModelIndex &parentIndex ) const
 }
 
 
-int DirTreeModel::columnCount( const QModelIndex &parent ) const
+int DirTreeModel::columnCount( const QModelIndex & parent ) const
 {
     Q_UNUSED( parent );
 
@@ -329,21 +345,19 @@ int DirTreeModel::columnCount( const QModelIndex &parent ) const
 }
 
 
-QVariant DirTreeModel::data( const QModelIndex &index, int role ) const
+QVariant DirTreeModel::data( const QModelIndex & index, int role ) const
 {
     if ( ! index.isValid() )
 	return QVariant();
 
-    DataColumn col = DataColumns::fromViewCol( index.column() );
+    DataColumn col  = DataColumns::fromViewCol( index.column() );
+    FileInfo * item = static_cast<FileInfo *>( index.internalPointer() );
+    CHECK_MAGIC( item );
 
     switch ( role )
     {
 	case Qt::DisplayRole:
 	    {
-		FileInfo * item = static_cast<FileInfo *>( index.internalPointer() );
-		CHECK_PTR( item );
-		CHECK_MAGIC( item );
-
 		QVariant result = columnText( item, col );
 
 		if ( item && item->isDirInfo() )
@@ -355,11 +369,16 @@ QVariant DirTreeModel::data( const QModelIndex &index, int role ) const
 		return result;
 	    }
 
+        case Qt::ForegroundRole:
+            {
+                if ( item->isIgnored() || item->isAttic() )
+                    return qAppPalette().brush( QPalette::Disabled, QPalette::Foreground );
+                else
+                    return QVariant();
+            }
+
 	case Qt::DecorationRole:
 	    {
-		FileInfo * item = static_cast<FileInfo *>( index.internalPointer() );
-		CHECK_MAGIC( item );
-
 		QVariant result = columnIcon( item, col );
 		return result;
 	    }
@@ -398,17 +417,14 @@ QVariant DirTreeModel::data( const QModelIndex &index, int role ) const
 
 	case RawDataRole: // Send raw data to our item delegate (the PercentBarDelegate)
 	    {
-		FileInfo * item = static_cast<FileInfo *>( index.internalPointer() );
-		CHECK_PTR( item );
-		CHECK_MAGIC( item );
-
 		switch ( col )
 		{
 		    case NameCol:	      return item->name();
 		    case PercentBarCol:
 			{
 			    if ( ( item->parent() && item->parent()->isBusy() ) ||
-				 item == _tree->firstToplevel() )
+				 item == _tree->firstToplevel() ||
+                                 item->isAttic() )
 			    {
 				return -1.0;
 			    }
@@ -455,7 +471,7 @@ QVariant DirTreeModel::headerData( int		   section,
 		case NameCol:		  return tr( "Name"		  );
 		case PercentBarCol:	  return tr( "Subtree Percentage" );
 		case PercentNumCol:	  return tr( "%"		  );
-		case SizeCol:	          return tr( "Size"	          );
+		case SizeCol:		  return tr( "Size"		  );
 		case TotalItemsCol:	  return tr( "Items"		  );
 		case TotalFilesCol:	  return tr( "Files"		  );
 		case TotalSubDirsCol:	  return tr( "Subdirs"		  );
@@ -476,7 +492,7 @@ QVariant DirTreeModel::headerData( int		   section,
 		case TotalItemsCol:
 		case TotalFilesCol:
 		case TotalSubDirsCol:
-                case LatestMTimeCol:
+		case LatestMTimeCol:
 		case PermissionsCol:
 		case OctalPermissionsCol: return Qt::AlignHCenter;
 		default:		  return Qt::AlignLeft;
@@ -488,14 +504,13 @@ QVariant DirTreeModel::headerData( int		   section,
 }
 
 
-Qt::ItemFlags DirTreeModel::flags( const QModelIndex &index ) const
+Qt::ItemFlags DirTreeModel::flags( const QModelIndex & index ) const
 {
     if ( ! index.isValid() )
 	return Qt::NoItemFlags;
 
 
     FileInfo * item = static_cast<FileInfo *>( index.internalPointer() );
-    CHECK_PTR( item );
     CHECK_MAGIC( item );
 
     Qt::ItemFlags baseFlags = Qt::ItemIsEnabled;
@@ -547,13 +562,12 @@ QModelIndex DirTreeModel::index( int row, int column, const QModelIndex & parent
 }
 
 
-QModelIndex DirTreeModel::parent( const QModelIndex &index ) const
+QModelIndex DirTreeModel::parent( const QModelIndex & index ) const
 {
     if ( ! index.isValid() )
 	return QModelIndex();
 
     FileInfo * child = static_cast<FileInfo*>( index.internalPointer() );
-    CHECK_PTR( child );
     CHECK_MAGIC( child );
 
     FileInfo * parent = child->parent();
@@ -642,14 +656,17 @@ QVariant DirTreeModel::columnText( FileInfo * item, int col ) const
     if ( col == _readJobsCol && item->isBusy() )
 	return tr( "[%1 Read Jobs]" ).arg( item->pendingReadJobs() );
 
-    bool limitedInfo = item->isDotEntry() || item->readState() == DirCached;
+    bool limitedInfo = item->isPseudoDir() || item->readState() == DirCached || item->isPkgInfo();
+
+    if ( item->isAttic() && col == PercentNumCol )
+        return QVariant();
 
     switch ( col )
     {
 	case NameCol:		  return item->name();
 	case PercentBarCol:	  return item->isExcluded() ? tr( "[Excluded]" ) : QVariant();
 	case PercentNumCol:	  return item == _tree->firstToplevel() ? QVariant() : formatPercent( item->subtreePercent() );
-        case SizeCol:	          return sizeColText( item );
+	case SizeCol:		  return sizeColText( item );
 	case LatestMTimeCol:	  return QString( "  " ) + formatTime( item->latestMtime() );
 	case UserCol:		  return limitedInfo ? QVariant() : item->userName();
 	case GroupCol:		  return limitedInfo ? QVariant() : item->groupName();
@@ -657,7 +674,7 @@ QVariant DirTreeModel::columnText( FileInfo * item, int col ) const
 	case OctalPermissionsCol: return limitedInfo ? QVariant() : item->octalPermissions();
     }
 
-    if ( item->isDirInfo() || item->isDotEntry() )
+    if ( item->isDirInfo() )
     {
 	QString prefix = item->readState() == DirAborted ? ">" : "";
 
@@ -677,15 +694,29 @@ QVariant DirTreeModel::columnText( FileInfo * item, int col ) const
 }
 
 
+int DirTreeModel::directChildrenCount( FileInfo * subtree ) const
+{
+    if ( ! subtree )
+	return 0;
+
+    int count = subtree->directChildrenCount();
+
+    if ( subtree->attic() )
+	++count;
+
+    return count;
+}
+
+
 QVariant DirTreeModel::sizeColText( FileInfo * item ) const
 {
     if ( item->isDevice() )
 	return QVariant();
 
-    if ( item->isDirInfo() || item->isDotEntry() )
+    if ( item->isDirInfo() )
     {
-        QString prefix = item->readState() == DirAborted ? ">" : "";
-        return prefix + formatSize( item->totalSize() );
+	QString prefix = item->readState() == DirAborted ? ">" : "";
+	return prefix + formatSize( item->totalSize() );
     }
 
     QString text;
@@ -732,6 +763,7 @@ QVariant DirTreeModel::columnIcon( FileInfo * item, int col ) const
     QPixmap icon;
 
     if	    ( item->isDotEntry() )  icon = _dotEntryIcon;
+    else if ( item->isPkgInfo()	 )  icon = _pkgIcon;
     else if ( item->isExcluded() )  icon = _excludedIcon;
     else if ( item->isDir()	 )
     {
@@ -761,8 +793,8 @@ void DirTreeModel::readJobFinished( DirInfo * dir )
 
     if ( anyAncestorBusy( dir ) )
     {
-        if  ( dir && ! dir->isMountPoint() )
-            logDebug() << "Ancestor busy - ignoring readJobFinished for " << dir << endl;
+	if  ( dir && ! dir->isMountPoint() )
+	    logDebug() << "Ancestor busy - ignoring readJobFinished for " << dir << endl;
     }
     else
     {
@@ -805,7 +837,7 @@ void DirTreeModel::newChildrenNotify( DirInfo * dir )
     }
 
     QModelIndex index = modelIndex( dir );
-    int count = countDirectChildren( dir );
+    int count = directChildrenCount( dir );
     // Debug::dumpDirectChildren( dir );
 
     if ( count > 0 )
@@ -950,6 +982,36 @@ void DirTreeModel::updatePersistentIndexes()
 }
 
 
+void DirTreeModel::beginRemoveRows( const QModelIndex & parent, int first, int last )
+{
+    if ( _removingRows )
+    {
+	logError() << "Removing rows already in progress" << endl;
+	return;
+    }
+
+
+    if ( ! parent.isValid() )
+    {
+	logError() << "Invalid QModelIndex" << endl;
+	return;
+    }
+
+    _removingRows = true;
+    QAbstractItemModel::beginRemoveRows( parent, first, last );
+}
+
+
+void DirTreeModel::endRemoveRows()
+{
+    if ( _removingRows )
+    {
+	QAbstractItemModel::endRemoveRows();
+	_removingRows = false;
+    }
+}
+
+
 void DirTreeModel::deletingChild( FileInfo * child )
 {
     logDebug() << "Deleting child " << child << endl;
@@ -970,23 +1032,22 @@ void DirTreeModel::deletingChild( FileInfo * child )
 
 void DirTreeModel::childDeleted()
 {
-    logDebug() << "endRemoveRows()" << endl;
     endRemoveRows();
 }
 
 
 void DirTreeModel::clearingSubtree( DirInfo * subtree )
 {
-    // logDebug() << "Deleting all children of " << subtree << endl;
+    logDebug() << "Deleting all children of " << subtree << endl;
 
     if ( subtree == _tree->root() || subtree->isTouched() )
     {
 	QModelIndex subtreeIndex = modelIndex( subtree, 0 );
-	int count = countDirectChildren( subtree );
+	int count = directChildrenCount( subtree );
 
 	if ( count > 0 )
 	{
-	    // logDebug() << "beginRemoveRows for " << subtree << " row 0 to " << count - 1 << endl;
+	    logDebug() << "beginRemoveRows for " << subtree << " row 0 to " << count - 1 << endl;
 	    beginRemoveRows( subtreeIndex, 0, count - 1 );
 	}
     }
@@ -999,7 +1060,6 @@ void DirTreeModel::subtreeCleared( DirInfo * subtree )
 {
     Q_UNUSED( subtree );
 
-    // logDebug() << "endRemoveRows()" << endl;
     endRemoveRows();
 }
 
