@@ -18,6 +18,7 @@
 
 #include "DataColumns.h"
 #include "FileInfo.h"
+#include "PkgFilter.h"
 
 
 namespace QDirStat
@@ -54,7 +55,7 @@ namespace QDirStat
 	virtual ~DirTreeModel();
 
 	/**
-	 * Returns the internal @ref DirTree this view works on.
+	 * Returns the internal DirTree this view works on.
 	 *
 	 * Handle with caution: This might be short-lived information.	The
 	 * model might choose to create a new tree shortly after returning
@@ -112,7 +113,7 @@ namespace QDirStat
 	 *
 	 * OK, I'm fed up with working around this poor design. Let's make our
 	 * own kludge to find out what items are expanded: Check the persistent
-	 * model indexes: The private class creates a persistend model index
+	 * model indexes: The private class creates a persistent model index
 	 * for each item that is expanded.
 	 **/
 	QModelIndexList persistentIndexList() const
@@ -123,7 +124,23 @@ namespace QDirStat
 	/**
 	 * Open a directory URL.
 	 **/
-	void openUrl( const QString &url );
+	void openUrl( const QString & url );
+
+        /**
+         * Open a pkg URL: Read installed packages that match the specified
+         * PkgFilter and their file lists from the system's package manager(s).
+         *
+         * Notice that PkgFilter has a constructor that takes a QString and
+         * uses PkgFilter::Auto as the default filter mode to determine the
+         * filter mode from any special characters present in the URL, e.g.
+         *
+         * "Pkg:/"                     -> PkgFilter::SelectAll
+         * contains "*" or "?"         -> PkgFilter::Wildcard
+         * contains "^" or "$" or ".*" -> PkgFilter::RegExp
+         * "Pkg:/=foo"                 -> PkgFilter::ExactMatch
+         * otherwise                   -> PkgFilter::StartsWith
+         **/
+        void readPkg( const PkgFilter & pkgFilter );
 
 	/**
 	 * Clear this view's contents.
@@ -176,6 +193,7 @@ namespace QDirStat
          **/
         bool slowUpdate() const { return _slowUpdate; }
 
+
     public:
 
 	// Mapping of tree items to model rows and vice versa.
@@ -197,11 +215,6 @@ namespace QDirStat
 	 **/
 	int rowNumber( FileInfo * child ) const;
 
-	/**
-	 * Count the direct children (including the "dot entry") of 'parent'.
-	 **/
-	int countDirectChildren( FileInfo * parent ) const;
-
 	/**
 	 * Return a model index for 'item' and 'column'.
 	 **/
@@ -249,7 +262,7 @@ namespace QDirStat
 	 * Return item flags for the specified model index. This specifies if
 	 * the item can be selected, edited etc.
 	 **/
-	virtual Qt::ItemFlags flags( const QModelIndex &index ) const Q_DECL_OVERRIDE;
+	virtual Qt::ItemFlags flags( const QModelIndex & index ) const Q_DECL_OVERRIDE;
 
 	/**
 	 * Return the model index for the specified row (direct tree child
@@ -270,6 +283,7 @@ namespace QDirStat
 	virtual void sort( int column,
 			   Qt::SortOrder order = Qt::AscendingOrder ) Q_DECL_OVERRIDE;
 
+
     protected slots:
 
 	/**
@@ -384,6 +398,12 @@ namespace QDirStat
 	 **/
 	QVariant columnIcon( FileInfo * item, int col ) const;
 
+        /**
+         * Return the number of direct children (plus the attic if there is
+         * one) of a subtree.
+         **/
+        int directChildrenCount( FileInfo * subtree ) const;
+
         /**
          * Return the text for the size for 'item'
          **/
@@ -395,6 +415,24 @@ namespace QDirStat
 	 **/
 	QVariant formatPercent( float percent ) const;
 
+        /**
+         * Start removing rows.
+         **/
+        void beginRemoveRows( const QModelIndex & parent, int first, int last );
+
+        /**
+         * End removing rows.
+         *
+         * Unlike the QAbstractItemModel's implementation, this method checks
+         * if removing rows is in progress in the first place so there will not
+         * be a segfault (!) if endRemoveRows is called without a corresponding
+         * beginRemoveRows().
+         *
+         * As usual, Qt's item classes don't even give it an honest try to do
+         * the most basic checking. This implementation does.
+         **/
+        void endRemoveRows();
+
 
 	//
 	// Data members
@@ -411,6 +449,7 @@ namespace QDirStat
         bool             _slowUpdate;
 	DataColumn	 _sortCol;
 	Qt::SortOrder	 _sortOrder;
+        bool             _removingRows;
 
 
 	// The various icons
@@ -426,6 +465,7 @@ namespace QDirStat
 	QPixmap _blockDeviceIcon;
 	QPixmap _charDeviceIcon;
 	QPixmap _specialIcon;
+        QPixmap _pkgIcon;
 
     };	// class DirTreeModel
 
