@@ -8,6 +8,9 @@
 
 
 #include "ExcludeRules.h"
+#include "DirInfo.h"
+#include "DotEntry.h"
+#include "FileInfoIterator.h"
 #include "Settings.h"
 #include "SettingsHelpers.h"
 #include "Logger.h"
@@ -19,17 +22,23 @@
 using namespace QDirStat;
 
 
-ExcludeRule::ExcludeRule( const QRegExp & regexp, bool useFullPath ):
+ExcludeRule::ExcludeRule( const QRegExp & regexp,
+                          bool            useFullPath,
+                          bool            checkAnyFileChild ):
     _regexp( regexp ),
-    _useFullPath( useFullPath )
+    _useFullPath( useFullPath ),
+    _checkAnyFileChild( checkAnyFileChild )
 {
     // NOP
 }
 
 
-ExcludeRule::ExcludeRule( const QString & regexp, bool useFullPath ):
+ExcludeRule::ExcludeRule( const QString & regexp,
+                          bool            useFullPath,
+                          bool            checkAnyFileChild ):
     _regexp( QRegExp( regexp ) ),
-    _useFullPath( useFullPath )
+    _useFullPath( useFullPath ),
+    _checkAnyFileChild( checkAnyFileChild )
 {
     // NOP
 }
@@ -43,6 +52,9 @@ ExcludeRule::~ExcludeRule()
 
 bool ExcludeRule::match( const QString & fullPath, const QString & fileName )
 {
+    if ( _checkAnyFileChild )  // use matchDirectChildren() for those rules
+        return false;
+
     QString matchText = _useFullPath ? fullPath : fileName;
 
     if ( matchText.isEmpty() )
@@ -55,6 +67,31 @@ bool ExcludeRule::match( const QString & fullPath, const QString & fileName )
 }
 
 
+bool ExcludeRule::matchDirectChildren( DirInfo * dir )
+{
+    if ( ! _checkAnyFileChild || ! dir )
+        return false;
+
+    if ( _regexp.pattern().isEmpty() )
+        return false;
+
+    FileInfoIterator it( dir->dotEntry() ? dir->dotEntry() : dir );
+
+    while ( *it )
+    {
+        if ( ! (*it)->isDir() )
+        {
+            if ( _regexp.exactMatch( (*it)->name() ) )
+                return true;
+        }
+
+        ++it;
+    }
+
+    return false;
+}
+
+
 //
 //---------------------------------------------------------------------------
 //
@@ -69,6 +106,26 @@ ExcludeRules::ExcludeRules():
 }
 
 
+ExcludeRules::ExcludeRules( const QStringList & paths ):
+    QObject(),
+    _listMover( _rules )
+{
+    _lastMatchingRule  = 0;
+    _defaultRulesAdded = false;
+
+    foreach ( const QString & path, paths )
+    {
+        QRegExp regexp( path );
+        regexp.setPatternSyntax( QRegExp::Wildcard );
+        ExcludeRule * rule = new ExcludeRule( regexp,
+                                              true,    // useFullPath
+                                              false ); // checkAnyFileChild
+        CHECK_NEW( rule );
+        _rules << rule;
+    }
+}
+
+
 ExcludeRules::~ExcludeRules()
 {
     clear();
@@ -104,18 +161,22 @@ void ExcludeRules::add( ExcludeRule * rule )
 }
 
 
-void ExcludeRules::add( const QRegExp & regexp, bool useFullPath )
+void ExcludeRules::add( const QRegExp & regexp,
+                        bool            useFullPath,
+                        bool            checkAnyFileChild )
 {
-    ExcludeRule * rule = new ExcludeRule( regexp, useFullPath );
+    ExcludeRule * rule = new ExcludeRule( regexp, useFullPath, checkAnyFileChild );
     CHECK_NEW( rule );
 
     instance()->add( rule );
 }
 
 
-void ExcludeRules::add( const QString & regexp, bool useFullPath )
+void ExcludeRules::add( const QString & regexp,
+                        bool            useFullPath,
+                        bool            checkAnyFileChild )
 {
-    add( QRegExp( regexp ), useFullPath );
+    add( QRegExp( regexp ), useFullPath, checkAnyFileChild );
 }
 
 
@@ -152,6 +213,30 @@ bool ExcludeRules::match( const QString & fullPath, const QString & fileName )
 }
 
 
+bool ExcludeRules::matchDirectChildren( DirInfo * dir )
+{
+    _lastMatchingRule = 0;
+    if ( ! dir )
+	return false;
+
+    foreach ( ExcludeRule * rule, _rules )
+    {
+	if ( rule->matchDirectChildren( dir ) )
+	{
+	    _lastMatchingRule = rule;
+#if VERBOSE_EXCLUDE_MATCHES
+
+	    logDebug() << dir << " matches " << rule << endl;
+
+#endif
+	    return true;
+	}
+    }
+
+    return false;
+}
+
+
 const ExcludeRule * ExcludeRules::matchingRule( const QString & fullPath,
 						const QString & fileName )
 {
@@ -223,9 +308,10 @@ void ExcludeRules::readSettings()
 
 	    // Read one exclude rule
 
-	    QString pattern    = settings.value( "Pattern"	       ).toString();
-	    bool caseSensitive = settings.value( "CaseSensitive", true ).toBool();
-	    bool useFullPath   = settings.value( "UseFullPath",	 false ).toBool();
+	    QString pattern        = settings.value( "Pattern"	                ).toString();
+	    bool caseSensitive     = settings.value( "CaseSensitive",     true  ).toBool();
+	    bool useFullPath       = settings.value( "UseFullPath",	  false ).toBool();
+            bool checkAnyFileChild = settings.value( "CheckAnyFileChild", false ).toBool();
 	    int syntax = readEnumEntry( settings, "Syntax",
 					QRegExp::RegExp,
 					patternSyntaxMapping() );
@@ -235,7 +321,7 @@ void ExcludeRules::readSettings()
 			    static_cast<QRegExp::PatternSyntax>( syntax ) );
 
 	    if ( ! pattern.isEmpty() && regexp.isValid() )
-		add( regexp, useFullPath );
+		add( regexp, useFullPath, checkAnyFileChild );
 	    else
 	    {
 		logError() << "Invalid regexp: \"" << regexp.pattern()
@@ -275,9 +361,10 @@ void ExcludeRules::writeSettings()
 	    groupName.sprintf( "ExcludeRule_%02d", i+1 );
 	    settings.beginGroup( groupName );
 
-	    settings.setValue( "Pattern",	regexp.pattern() );
-	    settings.setValue( "CaseSensitive", regexp.caseSensitivity() == Qt::CaseSensitive );
-	    settings.setValue( "UseFullPath",	rule->useFullPath() );
+	    settings.setValue( "Pattern",	    regexp.pattern() );
+	    settings.setValue( "CaseSensitive",     regexp.caseSensitivity() == Qt::CaseSensitive );
+	    settings.setValue( "UseFullPath",	    rule->useFullPath()       );
+	    settings.setValue( "CheckAnyFileChild", rule->checkAnyFileChild() );
 
 	    writeEnumEntry( settings, "Syntax",
 			    regexp.patternSyntax(),
