@@ -17,7 +17,10 @@
 
 #include "FileInfo.h"
 #include "DirInfo.h"
+#include "DotEntry.h"
+#include "Attic.h"
 #include "DirTree.h"
+#include "PkgInfo.h"
 #include "Logger.h"
 #include "Exception.h"
 
@@ -40,6 +43,7 @@ FileInfo::FileInfo( DirTree    * tree,
 {
     _isLocalFile  = true;
     _isSparseFile = false;
+    _isIgnored	  = false;
     _name	  = name ? name : "";
     _device	  = 0;
     _mode	  = 0;
@@ -64,6 +68,7 @@ FileInfo::FileInfo( const QString & filenameWithoutPath,
     CHECK_PTR( statInfo );
 
     _isLocalFile = true;
+    _isIgnored	  = false;
     _name	 = filenameWithoutPath;
 
     _device	 = statInfo->st_dev;
@@ -122,6 +127,7 @@ FileInfo::FileInfo( DirTree *	    tree,
 {
     _name	 = filenameWithoutPath;
     _isLocalFile = true;
+    _isIgnored	 = false;
     _device	 = 0;
     _mode	 = mode;
     _size	 = size;
@@ -195,7 +201,7 @@ QString FileInfo::url() const
     {
 	QString parentUrl = _parent->url();
 
-	if ( isDotEntry() )	// don't append "/." for dot entries
+	if ( isPseudoDir() ) // don't append "/." for dot entries and attics
 	    return parentUrl;
 
 	if ( ! parentUrl.endsWith( "/" ) && ! _name.startsWith( "/" ) )
@@ -208,14 +214,47 @@ QString FileInfo::url() const
 }
 
 
+QString FileInfo::path() const
+{
+    if ( isPkgInfo() )
+	return "";
+
+    if ( _parent )
+    {
+	QString parentPath = _parent->isPkgInfo() ? "/" : _parent->path();
+
+	if ( isPseudoDir() )
+	    return parentPath;
+
+	if ( ! parentPath.endsWith( "/" ) && ! _name.startsWith( "/" ) )
+	    parentPath += "/";
+
+	return parentPath + _name;
+    }
+    else
+	return _name;
+}
+
+
 QString FileInfo::debugUrl() const
 {
-    if ( this == _tree->root() )
+    if ( _tree && this == _tree->root() )
 	return "<root>";
 
     if ( isDotEntry() )
 	return url() + "/" + dotEntryName();
 
+    if ( isAttic() )
+    {
+        if ( _parent )
+        {
+            if ( _tree && _parent != _tree->root() )
+                return _parent->debugUrl() + "/" + atticName();
+        }
+
+        return url() + "/" + atticName();
+    }
+
     return url() ;
 }
 
@@ -257,8 +296,13 @@ bool FileInfo::isInSubtree( const FileInfo *subtree ) const
 }
 
 
-FileInfo * FileInfo::locate( QString url, bool findDotEntries )
+FileInfo * FileInfo::locate( QString url, bool findPseudoDirs )
 {
+    if ( ! _tree )
+	return 0;
+
+    FileInfo * result = 0;
+
     if ( ! url.startsWith( _name ) && this != _tree->root() )
 	return 0;
     else					// URL starts with this node's name
@@ -270,24 +314,26 @@ FileInfo * FileInfo::locate( QString url, bool findDotEntries )
 	    if ( url.length() == 0 )		// Nothing left?
 		return this;			// Hey! That's us!
 
-	    if ( url.startsWith( "/" ) )	// If the next thing a path delimiter,
+	    if ( url.startsWith( "/" ) )	// If the next thing is a path delimiter,
 		url.remove( 0, 1 );		// remove that leading delimiter.
 	    else				// No path delimiter at the beginning
 	    {
 		if ( _name.right(1) != "/" &&	// and this is not the root directory
 		     ! isDotEntry() )		// or a dot entry:
+		{
 		    return 0;			// This can't be any of our children.
+		}
 	    }
 	}
 
 
 	// Search all children
 
-	FileInfo *child = firstChild();
+	FileInfo * child = firstChild();
 
 	while ( child )
 	{
-	    FileInfo *foundChild = child->locate( url, findDotEntries );
+	    FileInfo * foundChild = child->locate( url, findPseudoDirs );
 
 	    if ( foundChild )
 		return foundChild;
@@ -296,10 +342,22 @@ FileInfo * FileInfo::locate( QString url, bool findDotEntries )
 	}
 
 
-	// Special case: The dot entry is requested.
+	// Special case: One of the pseudo directories is requested.
 
-	if ( findDotEntries && dotEntry() && url == dotEntryName() )
-	    return dotEntry();
+	if ( findPseudoDirs )
+	{
+	    if ( dotEntry() && url == dotEntryName() )
+		return dotEntry();
+
+	    if ( attic() && url == atticName() )
+		return attic();
+
+            if ( url == dotEntryName() + "/" + atticName() &&
+                 dotEntry() && dotEntry()->attic() )
+            {
+                return dotEntry()->attic();
+            }
+	}
 
 	// Search the dot entry if there is one - but only if there is no more
 	// path delimiter left in the URL. The dot entry contains files only,
@@ -311,11 +369,14 @@ FileInfo * FileInfo::locate( QString url, bool findDotEntries )
 	if ( dotEntry() &&
 	     ! url.contains( "/" ) )	   // No (more) "/" in this URL
 	{
-	    return dotEntry()->locate( url, findDotEntries );
+	    result = dotEntry()->locate( url, findPseudoDirs );
 	}
+
+	if ( ! result && attic() )
+	    result = attic()->locate( url, findPseudoDirs );
     }
 
-    return 0;
+    return result;
 }
 
 
@@ -339,24 +400,30 @@ QString FileInfo::dotEntryName()
 }
 
 
+QString FileInfo::atticName()
+{
+    return QObject::tr( "<Ignored>" );
+}
+
+
 bool FileInfo::isCached() const
 {
-    if ( isDirInfo() && ! isDotEntry() )
-        return readState() == DirCached;
+    if ( isDirInfo() && ! isPseudoDir() )
+	return readState() == DirCached;
     else
-        return _parent && _parent->readState() == DirCached;
+	return _parent && _parent->readState() == DirCached;
 }
 
 
 bool FileInfo::hasUid() const
 {
-    return ! isCached();
+    return ! isPkgInfo() && ! isCached();
 }
 
 
 bool FileInfo::hasGid() const
 {
-    return ! isCached();
+    return ! isPkgInfo() && ! isCached();
 }
 
 
@@ -461,6 +528,46 @@ DirInfo * FileInfo::toDirInfo()
 }
 
 
+DotEntry * FileInfo::toDotEntry()
+{
+    DotEntry * dotEntry = dynamic_cast<DotEntry *>( this );
+
+    return dotEntry;
+}
+
+
+Attic * FileInfo::toAttic()
+{
+    Attic * attic = dynamic_cast<Attic *>( this );
+
+    return attic;
+}
+
+
+PkgInfo * FileInfo::toPkgInfo()
+{
+    PkgInfo * pkgInfo = dynamic_cast<PkgInfo *>( this );
+
+    return pkgInfo;
+}
+
+
+PkgInfo * FileInfo::pkgInfoParent() const
+{
+    FileInfo * pkg = _parent;
+
+    while ( pkg )
+    {
+	if ( pkg->isPkgInfo() )
+	    return pkg->toPkgInfo();
+
+	pkg = pkg->parent();
+    }
+
+    return 0;
+}
+
+
 QString QDirStat::formatTime( time_t rawTime )
 {
     if ( rawTime == (time_t) 0 )
