@@ -32,6 +32,9 @@ namespace QDirStat
 
     // Forward declarations
     class DirInfo;
+    class DotEntry;
+    class Attic;
+    class PkgInfo;
     class DirTree;
 
 
@@ -51,7 +54,7 @@ namespace QDirStat
 
 
     /**
-     * The most basic building block of a @ref DirTree:
+     * The most basic building block of a DirTree:
      *
      * Information about one single directory entry. This is the type of info
      * typically obtained by stat() / lstat() or similar calls.
@@ -85,7 +88,7 @@ namespace QDirStat
 		  DirInfo	* parent = 0 );
 
 	/**
-	 * Constructor from the bare neccessary fields
+	 * Constructor from the bare necessary fields
 	 * for use from a cache file reader
 	 *
 	 * If 'blocks' is -1, it will be calculated from 'size'.
@@ -102,7 +105,7 @@ namespace QDirStat
 	/**
 	 * Destructor.
 	 *
-	 * Don't forget to call @ref FileInfo::unlinkChild() when deleting
+	 * Don't forget to call FileInfo::unlinkChild() when deleting
 	 * objects of this class!
 	 **/
 	virtual ~FileInfo();
@@ -134,11 +137,11 @@ namespace QDirStat
 	 **/
 	QString name() const { return _name; }
 
-        /**
-         * Returns the base name of this object, i.e. the last path component,
-         * even if this is a toplevel item.
-         **/
-        QString baseName() const;
+	/**
+	 * Returns the base name of this object, i.e. the last path component,
+	 * even if this is a toplevel item.
+	 **/
+	QString baseName() const;
 
 	/**
 	 * Returns the full URL of this object with full path.
@@ -146,17 +149,32 @@ namespace QDirStat
 	 * This is a (somewhat) expensive operation since it will recurse up
 	 * to the top of the tree.
 	 **/
-	QString url() const;
+	virtual QString url() const;
 
 	/**
-	 * Very much like @ref FileInfo::url(), but with "/<Files>" appended
-	 * if this is a dot entry. Useful for debugging.
-	 * Notice: You can simply use the @ref QTextStream operator<< to
-	 * output exactly this:
+	 * Returns the full path of this object. Unlike url(), this never has a
+	 * protocol prefix or a part that identifies the package this belongs
+	 * to. This is the path that can be used to find this object in the
+	 * filesystem.
+	 *
+	 * url()  might return	"Pkg:/chromium-browser/usr/lib/chromium/foo.z"
+	 * path() returns just	"/usr/lib/chromium/foo.z"
+	 *
+	 * Like url(), this is somewhat expensive since it recurses up the
+	 * tree, but it stops when a PkgInfo node is found there.
+	 **/
+	virtual QString path() const;
+
+	/**
+	 * Very much like FileInfo::url(), but with "/<Files>" appended if this
+	 * is a dot entry. Useful for debugging.
+	 *
+	 * Notice: You can simply use the QTextStream operator<< to output
+	 * exactly this:
 	 *
 	 * logDebug() << "Found fileInfo " << info << endl;
 	 **/
-	QString debugUrl() const;
+	virtual QString debugUrl() const;
 
 	/**
 	 * Returns the major and minor device numbers of the device this file
@@ -167,9 +185,9 @@ namespace QDirStat
 	/**
 	 * The file permissions and object type as returned by lstat().
 	 * You might want to use the repective convenience methods instead:
-	 * @ref isDir(), @ref isFile(), ...
-         *
-         * See also @ref symbolicPermissions(), octalPermissions()
+	 * isDir(), isFile(), ...
+	 *
+	 * See also symbolicPermissions(), octalPermissions()
 	 **/
 	mode_t mode() const { return _mode;   }
 
@@ -195,13 +213,13 @@ namespace QDirStat
 	 **/
 	QString userName() const;
 
-        /**
-         * Return 'true' if this FileInfo has a UID (user ID).x
-         *
-         * It might not have that information e.g. if it was read from a cache
-         * file.
-         **/
-        bool hasUid() const;
+	/**
+	 * Return 'true' if this FileInfo has a UID (user ID).
+	 *
+	 * It might not have that information e.g. if it was read from a cache
+	 * file.
+	 **/
+	bool hasUid() const;
 
 	/**
 	 * Group ID of the owner.
@@ -219,25 +237,25 @@ namespace QDirStat
 	 **/
 	QString groupName() const;
 
-        /**
-         * Return 'true' if this FileInfo has a GID (group ID).
-         *
-         * It might not have that information e.g. if it was read from a cache
-         * file.
-         **/
-        bool hasGid() const;
+	/**
+	 * Return 'true' if this FileInfo has a GID (group ID).
+	 *
+	 * It might not have that information e.g. if it was read from a cache
+	 * file.
+	 **/
+	bool hasGid() const;
 
-        /**
-         * File permissions formatted like in "ls -l", i.e. "-rwxrwxrwx",
-         * "drwxrwxrwx"
-         **/
-        QString symbolicPermissions() const;
+	/**
+	 * File permissions formatted like in "ls -l", i.e. "-rwxrwxrwx",
+	 * "drwxrwxrwx"
+	 **/
+	QString symbolicPermissions() const;
 
-        /**
-         * File permissions formatted as octal number (like used by the "chmod"
-         * command, i.e. "0644"
-         **/
-        QString octalPermissions() const;
+	/**
+	 * File permissions formatted as octal number (like used by the "chmod"
+	 * command, i.e. "0644"
+	 **/
+	QString octalPermissions() const;
 
 	/**
 	 * The file size in bytes. This does not take unused space in the last
@@ -249,14 +267,14 @@ namespace QDirStat
 
 	/**
 	 * The number of bytes actually allocated on the file system. Usually
-	 * this will be more than @ref byteSize() since the last few bytes of a
-	 * file usually consume an additional cluster on the file system.
+	 * this will be more than byteSize() since the last few bytes of a file
+	 * usually consume an additional cluster on the file system.
 	 *
 	 * In the case of sparse files, however, this might as well be
-	 * considerably less than @ref byteSize() - this means that this file
-	 * has "holes", i.e. large portions filled with zeros. This is typical
-	 * for large core dumps for example. The only way to create such a file
-	 * is to lseek() far ahead of the previous file size and then writing
+	 * considerably less than byteSize() - this means that this file has
+	 * "holes", i.e. large portions filled with zeros. This is typical for
+	 * large core dumps for example. The only way to create such a file is
+	 * to lseek() far ahead of the previous file size and then writing
 	 * data. Most file system utilities will however disregard the fact
 	 * that files are sparse files and simply allocate the holes as well,
 	 * thus greatly increasing the disk space consumption of such a
@@ -280,7 +298,7 @@ namespace QDirStat
 	FileSize blocks() const { return _blocks; }
 
 	/**
-	 * The size of one single block that @ref blocks() returns.
+	 * The size of one single block that blocks() returns.
 	 * Notice: This is _not_ the blocksize that lstat() returns!
 	 **/
 	FileSize blockSize() const { return 512L;    }
@@ -323,6 +341,35 @@ namespace QDirStat
 	 **/
 	virtual int totalFiles() { return 0; }
 
+	/**
+	 * Returns the total number of non-directory items in this subtree,
+	 * excluding this item.
+	 * Derived classes that have children should overwrite this.
+	 **/
+	virtual int totalNonDirItems() { return 0; }
+
+	/**
+	 * Returns the total number of ignored (non-directory!) items in this
+	 * subtree, excluding this item.
+	 * Derived classes that have children should overwrite this.
+	 **/
+	virtual int totalIgnoredItems() { return 0; }
+
+	/**
+	 * Returns the total number of not ignored (non-directory!) items in
+	 * this subtree, excluding this item.
+	 *
+	 * Derived classes that have children should overwrite this.
+	 **/
+	virtual int totalUnignoredItems() { return 0; }
+
+	/**
+	 * Returns the total number of direct children of this item.
+	 *
+	 * Derived classes that have children should overwrite this.
+	 **/
+	virtual int directChildrenCount() { return 0; }
+
 	/**
 	 * Returns the latest modification time of this subtree.
 	 * Derived classes that have children should overwrite this.
@@ -399,7 +446,7 @@ namespace QDirStat
 	//
 
 	/**
-	 * Returns a pointer to the @ref DirTree this entry belongs to.
+	 * Returns a pointer to the DirTree this entry belongs to.
 	 **/
 	DirTree * tree() const { return _tree; }
 
@@ -465,10 +512,10 @@ namespace QDirStat
 	 * it's only advisable to do so if a derived class comes up with a
 	 * different method than brute-force searching all children.
 	 *
-	 * 'findDotEntries' specifies if locating "dot entries" (".../<Files>")
-	 * is desired.
+	 * 'findPseudoDirs' specifies if locating pseudo directories like "dot
+	 * entries" (".../<Files>") or "attics" (".../<Ignored>") is desired.
 	 **/
-	virtual FileInfo * locate( QString url, bool findDotEntries = false );
+	virtual FileInfo * locate( QString url, bool findPseudoDirs = false );
 
 	/**
 	 * Insert a child into the children list.
@@ -489,7 +536,7 @@ namespace QDirStat
 	 *
 	 * This default implementation always returns 0.
 	 **/
-	virtual FileInfo *dotEntry()	const { return 0; }
+	virtual DotEntry * dotEntry() const { return 0; }
 
 	/**
 	 * Set a "Dot Entry". This makes sense for directories only.
@@ -499,19 +546,49 @@ namespace QDirStat
 	virtual void setDotEntry( FileInfo *newDotEntry )
 	    { Q_UNUSED( newDotEntry ); }
 
+	/**
+	 * Return 'true' if this is a pseudo directory: A "dot entry" or an
+	 * "attic".
+	 **/
+	virtual bool isPseudoDir() const
+	    { return isDotEntry() || isAttic(); }
+
 	/**
 	 * Returns true if this is a "Dot Entry".
-	 * See @ref dotEntry() for details.
+	 * See dotEntry() for details.
 	 *
 	 * This default implementation always returns false.
 	 **/
 	virtual bool isDotEntry() const { return false; }
 
+	/**
+	 * Return the "Attic" entry for this node if there is one (or 0
+	 * otherwise): This is a pseudo entry that directory nodes use to store
+	 * ignored files and directories separately from the normal tree
+	 * hierarchy.
+	 *
+	 * This default implementation always returns 0.
+	 **/
+	virtual Attic * attic() const { return 0; }
+
+	/**
+	 * Check if this is an attic entry where ignored files and directories
+	 * are stored.
+	 *
+	 * This default implementation always returns false.
+	 **/
+	virtual bool isAttic() const { return false; }
+
 	/**
 	 * (Translated) user-visible string for a "Dot Entry" ("<Files>").
 	 **/
 	static QString dotEntryName();
 
+	/**
+	 * (Translated) user-visible string for the "Attic" ("<Ignored>").
+	 **/
+	static QString atticName();
+
 	/**
 	 * Returns the tree level (depth) of this item.
 	 * The topmost level is 0.
@@ -556,24 +633,49 @@ namespace QDirStat
 	virtual DirReadState readState() const { return DirFinished; }
 
 	/**
-	 * Returns true if this is a @ref DirInfo object.
+	 * Returns true if this is a DirInfo object.
 	 *
-	 * Don't confuse this with @ref isDir() which tells whether or not this
-	 * is a disk directory! Both should return the same, but you'll never
-	 * know - better be safe than sorry!
+	 * Don't confuse this with isDir() which tells whether or not this is a
+	 * disk directory! Both should return the same, but you'll never know -
+	 * better be safe than sorry!
 	 *
 	 * This default implementation always returns 'false'. Derived classes
-	 * (in particular, those derived from @ref DirInfo) should overwrite
-	 * this.
+	 * (in particular, those derived from DirInfo) should overwrite this.
 	 **/
 	virtual bool isDirInfo() const { return false; }
 
 	/**
-	 * Try to convert this to a @ref DirInfo pointer. This returns null if
-	 * this is not a DirInfo.
+	 * Returns true if this is a PkgInfo object.
+	 *
+	 * This default implementation always returns 'false'. Derived classes
+	 * (in particular, those derived from PkgInfo) should overwrite this.
+	 **/
+	virtual bool isPkgInfo() const { return false; }
+
+	/**
+	 * Try to convert this to a DirInfo pointer. This returns null if this
+	 * is not a DirInfo.
 	 **/
 	DirInfo * toDirInfo();
 
+	/**
+	 * Try to convert this to a DirInfo pointer. This returns null if this
+	 * is not a DirInfo.
+	 **/
+	DotEntry * toDotEntry();
+
+	/**
+	 * Try to convert this to a DirInfo pointer. This returns null if this
+	 * is not a DirInfo.
+	 **/
+	Attic * toAttic();
+
+	/**
+	 * Try to convert this to a PkgInfo pointer. This returns null if this
+	 * is not a DirInfo.
+	 **/
+	PkgInfo * toPkgInfo();
+
 	/**
 	 * Returns true if this is a sparse file, i.e. if this file has
 	 * actually fewer disk blocks allocated than its byte size would call
@@ -583,14 +685,31 @@ namespace QDirStat
 	 * calculated in the constructor rather than doing repeated
 	 * calculations and comparisons.
 	 *
-	 * Please not that @ref size() already takes this into account.
+	 * Please not that size() already takes this into account.
 	 **/
 	bool isSparseFile() const { return _isSparseFile; }
 
-        /**
-         * Returns true if this FileInfo was read from a cache file.
-         **/
-        bool isCached() const;
+	/**
+	 * Returns true if this FileInfo was read from a cache file.
+	 **/
+	bool isCached() const;
+
+	/**
+	 * Returns true if this FileInfo was ignored by some rule (e.g. in the
+	 * "unpackaged files" view).
+	 **/
+	bool isIgnored() const { return _isIgnored; }
+
+	/**
+	 * Set the "ignored" flag. Notice that this only sets the flag; it does
+	 * not reparent the FileInfo or anything like that.
+	 **/
+	void setIgnored( bool ignored ) { _isIgnored = ignored; }
+
+	/**
+	 * Return the nearest PkgInfo parent or 0 if there is none.
+	 **/
+	PkgInfo * pkgInfoParent() const;
 
 
 	//
@@ -661,6 +780,7 @@ namespace QDirStat
 	QString		_name;			// the file name (without path!)
 	bool		_isLocalFile  :1;	// flag: local or remote file?
 	bool		_isSparseFile :1;	// (cache) flag: sparse file (file with "holes")?
+	bool		_isIgnored    :1;	// flag: ignored by rule?
 	dev_t		_device;		// device this object resides on
 	mode_t		_mode;			// file permissions + object type
 	nlink_t		_links;			// number of links
@@ -719,7 +839,7 @@ namespace QDirStat
     /**
      * Format the mode (the permissions bits) returned from the stat() system
      * call in the commonly used formats, both symbolic and octal, e.g.
-     *     drwxr-xr-x  0755
+     *	   drwxr-xr-x  0755
      **/
     QString formatPermissions( mode_t mode );
 
@@ -732,7 +852,7 @@ namespace QDirStat
      * Return the mode (the permission bits) returned from stat() like the
      * "ls -l" shell command does, e.g.
      *
-     *     drwxr-xr-x
+     *	   drwxr-xr-x
      *
      * 'omitTypeForRegularFiles' specifies if the leading "-" should be omitted.
      **/
@@ -746,7 +866,7 @@ namespace QDirStat
 
 
     /**
-     * Print the debugUrl() of a @ref FileInfo in a debug stream.
+     * Print the debugUrl() of a FileInfo in a debug stream.
      **/
     inline QTextStream & operator<< ( QTextStream & stream, const FileInfo * info )
     {
