@@ -10,13 +10,13 @@
 #include <QApplication>
 #include <QCloseEvent>
 #include <QMessageBox>
-#include <QProcess>
 #include <QFileDialog>
 #include <QSignalMapper>
 #include <QClipboard>
 
 #include "MainWindow.h"
 #include "ActionManager.h"
+#include "BusyPopup.h"
 #include "CleanupCollection.h"
 #include "CleanupConfigPage.h"
 #include "ConfigDialog.h"
@@ -25,6 +25,8 @@
 #include "DirTree.h"
 #include "DirTreeCache.h"
 #include "DirTreeModel.h"
+#include "DirTreePatternFilter.h"
+#include "DirTreePkgFilter.h"
 #include "Exception.h"
 #include "ExcludeRules.h"
 #include "FileDetailsView.h"
@@ -34,17 +36,28 @@
 #include "Logger.h"
 #include "MimeCategorizer.h"
 #include "MimeCategoryConfigPage.h"
+#include "OpenPkgDialog.h"
 #include "OutputWindow.h"
+#include "PkgManager.h"
+#include "PkgQuery.h"
 #include "Refresher.h"
 #include "SelectionModel.h"
 #include "Settings.h"
 #include "SettingsHelpers.h"
+#include "ShowUnpkgFilesDialog.h"
 #include "SysUtil.h"
 #include "Trash.h"
 #include "Version.h"
 
-#define LONG_MESSAGE	25*1000
-#define UPDATE_MILLISEC 200
+#define LONG_MESSAGE		25*1000
+#define UPDATE_MILLISEC		200
+
+#if (QT_VERSION < QT_VERSION_CHECK( 5, 13, 0 ))
+#  define HAVE_SIGNAL_MAPPER	  1
+#else
+// QSignalMapper is deprecated from Qt 5.13 on
+#  define HAVE_SIGNAL_MAPPER	  0
+#endif
 
 using namespace QDirStat;
 
@@ -52,6 +65,7 @@ using QDirStat::DataColumns;
 using QDirStat::DirTreeModel;
 using QDirStat::SelectionModel;
 using QDirStat::CleanupCollection;
+using QDirStat::PkgFilter;
 
 
 MainWindow::MainWindow():
@@ -89,7 +103,6 @@ MainWindow::MainWindow():
     _ui->treemapView->setSelectionModel( _selectionModel );
 
     _dirTreeModel->setSelectionModel( _selectionModel );
-    changeLayout( _layoutName );
 
     _cleanupCollection = new CleanupCollection( _selectionModel );
     CHECK_NEW( _cleanupCollection );
@@ -103,10 +116,7 @@ MainWindow::MainWindow():
     _ui->dirTreeView->setCleanupCollection( _cleanupCollection );
     _ui->treemapView->setCleanupCollection( _cleanupCollection );
 
-    _mimeCategorizer = new MimeCategorizer();
-    CHECK_NEW( _mimeCategorizer );
-
-    _ui->treemapView->setMimeCategorizer( _mimeCategorizer );
+    _ui->breadcrumbNavigator->clear();
 
 #ifdef Q_OS_MACX
     // This makes the application to look like more "native" on macOS
@@ -116,6 +126,28 @@ MainWindow::MainWindow():
 
     connectSignals();
     connectActions();
+    changeLayout( _layoutName );
+
+    if ( ! PkgQuery::haveGetInstalledPkgSupport() ||
+	 ! PkgQuery::haveFileListSupport()	    )
+    {
+	logInfo() << "No package manager support "
+		  << "for getting installed packages or file lists"
+		  << endl;
+
+	_ui->actionOpenPkg->setEnabled( false );
+    }
+
+    PkgManager * pkgManager = PkgQuery::primaryPkgManager();
+
+    if ( ! pkgManager || ! pkgManager->supportsFileListCache() )
+    {
+	logInfo() << "No package manager support "
+		  << "for getting a file lists cache"
+		  << endl;
+
+	_ui->actionShowUnpkgFiles->setEnabled( false );
+    }
 
     if ( ! _ui->actionShowTreemap->isChecked() )
 	_ui->treemapView->disable();
@@ -132,6 +164,7 @@ MainWindow::~MainWindow()
 
     writeSettings();
     ExcludeRules::instance()->writeSettings();
+    MimeCategorizer::instance()->writeSettings();
 
     // Relying on the QObject hierarchy to properly clean this up resulted in a
     //	segfault; there was probably a problem in the deletion order.
@@ -141,7 +174,6 @@ MainWindow::~MainWindow()
 
     delete _ui->dirTreeView;
     delete _cleanupCollection;
-    delete _mimeCategorizer;
     delete _selectionModel;
     delete _dirTreeModel;
 
@@ -217,20 +249,24 @@ void MainWindow::connectActions()
 {
     // "File" menu
 
-    CONNECT_ACTION( _ui->actionOpen,			    this, askOpenUrl()	    );
-    CONNECT_ACTION( _ui->actionRefreshAll,		    this, refreshAll()	    );
-    CONNECT_ACTION( _ui->actionRefreshSelected,		    this, refreshSelected() );
-    CONNECT_ACTION( _ui->actionReadExcludedDirectory,	    this, refreshSelected() );
-    CONNECT_ACTION( _ui->actionContinueReadingAtMountPoint, this, refreshSelected() );
-    CONNECT_ACTION( _ui->actionStopReading,		    this, stopReading()	    );
-    CONNECT_ACTION( _ui->actionAskWriteCache,		    this, askWriteCache()   );
-    CONNECT_ACTION( _ui->actionAskReadCache,		    this, askReadCache()    );
-    CONNECT_ACTION( _ui->actionQuit,			    qApp, quit()	    );
+    CONNECT_ACTION( _ui->actionOpenDir,			    this, askOpenDir()	      );
+    CONNECT_ACTION( _ui->actionOpenPkg,			    this, askOpenPkg()	      );
+    CONNECT_ACTION( _ui->actionShowUnpkgFiles,		    this, askShowUnpkgFiles() );
+    CONNECT_ACTION( _ui->actionRefreshAll,		    this, refreshAll()	      );
+    CONNECT_ACTION( _ui->actionRefreshSelected,		    this, refreshSelected()   );
+    CONNECT_ACTION( _ui->actionReadExcludedDirectory,	    this, refreshSelected()   );
+    CONNECT_ACTION( _ui->actionContinueReadingAtMountPoint, this, refreshSelected()   );
+    CONNECT_ACTION( _ui->actionStopReading,		    this, stopReading()	      );
+    CONNECT_ACTION( _ui->actionAskWriteCache,		    this, askWriteCache()     );
+    CONNECT_ACTION( _ui->actionAskReadCache,		    this, askReadCache()      );
+    CONNECT_ACTION( _ui->actionQuit,			    qApp, quit()	      );
 
 
 
     // "View" menu
 
+#if HAVE_SIGNAL_MAPPER
+
     _treeLevelMapper = new QSignalMapper( this );
 
     connect( _treeLevelMapper, SIGNAL( mapped		( int ) ),
@@ -249,6 +285,23 @@ void MainWindow::connectActions()
 
     mapTreeExpandAction( _ui->actionCloseAllTreeLevels, 0 );
 
+#else
+
+    connect( _ui->actionExpandTreeLevel0,   &QAction::triggered, [=]() { expandTreeToLevel( 0 ); } );
+    connect( _ui->actionExpandTreeLevel1,   &QAction::triggered, [=]() { expandTreeToLevel( 1 ); } );
+    connect( _ui->actionExpandTreeLevel2,   &QAction::triggered, [=]() { expandTreeToLevel( 2 ); } );
+    connect( _ui->actionExpandTreeLevel3,   &QAction::triggered, [=]() { expandTreeToLevel( 3 ); } );
+    connect( _ui->actionExpandTreeLevel4,   &QAction::triggered, [=]() { expandTreeToLevel( 4 ); } );
+    connect( _ui->actionExpandTreeLevel5,   &QAction::triggered, [=]() { expandTreeToLevel( 5 ); } );
+    connect( _ui->actionExpandTreeLevel6,   &QAction::triggered, [=]() { expandTreeToLevel( 6 ); } );
+    connect( _ui->actionExpandTreeLevel7,   &QAction::triggered, [=]() { expandTreeToLevel( 7 ); } );
+    connect( _ui->actionExpandTreeLevel8,   &QAction::triggered, [=]() { expandTreeToLevel( 8 ); } );
+    connect( _ui->actionExpandTreeLevel9,   &QAction::triggered, [=]() { expandTreeToLevel( 9 ); } );
+
+    connect( _ui->actionCloseAllTreeLevels, &QAction::triggered, [=]() { expandTreeToLevel( 0 ); } );
+
+#endif
+
     connect( _ui->actionShowCurrentPath,  SIGNAL( toggled   ( bool ) ),
 	     _ui->breadcrumbNavigator,	  SLOT	( setVisible( bool ) ) );
 
@@ -268,8 +321,8 @@ void MainWindow::connectActions()
 
     // "Edit" menu
 
-    CONNECT_ACTION( _ui->actionCopyUrlToClipboard, this, copyCurrentUrlToClipboard() );
-    CONNECT_ACTION( _ui->actionMoveToTrash,	   this, moveToTrash() );
+    CONNECT_ACTION( _ui->actionCopyPathToClipboard, this, copyCurrentPathToClipboard() );
+    CONNECT_ACTION( _ui->actionMoveToTrash,	    this, moveToTrash() );
 
 
     // "Go To" menu
@@ -298,11 +351,13 @@ void MainWindow::connectActions()
 
     // "Help" menu
 
-    CONNECT_ACTION( _ui->actionHelp,	    this, showHelp() );
-    CONNECT_ACTION( _ui->actionAbout,	    this, showAboutDialog() );
-    CONNECT_ACTION( _ui->actionAboutQt,	    qApp, aboutQt() );
-    CONNECT_ACTION( _ui->actionWhatsNew,    this, showWhatsNew() );
-    CONNECT_ACTION( _ui->actionDonate,	    this, showDonateDialog() );
+    CONNECT_ACTION( _ui->actionHelp,		this, showHelp()          );
+    CONNECT_ACTION( _ui->actionPkgViewHelp,	this, showPkgViewHelp()   );
+    CONNECT_ACTION( _ui->actionUnpkgViewHelp,	this, showUnpkgViewHelp() );
+    CONNECT_ACTION( _ui->actionAbout,		this, showAboutDialog()   );
+    CONNECT_ACTION( _ui->actionAboutQt,		qApp, aboutQt()           );
+    CONNECT_ACTION( _ui->actionWhatsNew,	this, showWhatsNew()      );
+    CONNECT_ACTION( _ui->actionDonate,		this, showDonateDialog()  );
 
 
     // Invisible debug actions
@@ -319,38 +374,44 @@ void MainWindow::connectActions()
 
 void MainWindow::mapTreeExpandAction( QAction * action, int level )
 {
-    CONNECT_ACTION( action, _treeLevelMapper, map() );
-    _treeLevelMapper->setMapping( action, level );
+    if ( _treeLevelMapper )
+    {
+	CONNECT_ACTION( action, _treeLevelMapper, map() );
+	_treeLevelMapper->setMapping( action, level );
+    }
 }
 
 
 void MainWindow::updateActions()
 {
-    bool reading = _dirTreeModel->tree()->isBusy();
+    bool reading	     = _dirTreeModel->tree()->isBusy();
+    FileInfo * currentItem   = _selectionModel->currentItem();
+    FileInfo * firstToplevel = _dirTreeModel->tree()->firstToplevel();
+    bool pkgView	     = firstToplevel && firstToplevel->isPkgInfo();
 
     _ui->actionStopReading->setEnabled( reading );
     _ui->actionRefreshAll->setEnabled	( ! reading );
     _ui->actionAskReadCache->setEnabled ( ! reading );
     _ui->actionAskWriteCache->setEnabled( ! reading );
 
-    bool haveCurrentItem = ( _selectionModel->currentItem() != 0 );
-    bool treeNotEmpty	 = ( _dirTreeModel->tree()->firstToplevel() != 0 );
-
-    _ui->actionCopyUrlToClipboard->setEnabled( haveCurrentItem );
-    _ui->actionGoUp->setEnabled( haveCurrentItem );
-    _ui->actionGoToToplevel->setEnabled( treeNotEmpty );
+    _ui->actionCopyPathToClipboard->setEnabled( currentItem );
+    _ui->actionGoUp->setEnabled( currentItem && currentItem->treeLevel() > 1 );
+    _ui->actionGoToToplevel->setEnabled( firstToplevel && ( ! currentItem || currentItem->treeLevel() > 1 ));
 
     FileInfoSet selectedItems = _selectionModel->selectedItems();
-    FileInfo * sel = selectedItems.first();
-    bool oneDirSelected = ( selectedItems.size() == 1 ) && sel && sel->isDir();
+    FileInfo * sel	      = selectedItems.first();
+    int selSize		      = selectedItems.size();
 
-    _ui->actionMoveToTrash->setEnabled( sel && ! sel->isDotEntry() && ! reading );
-    _ui->actionRefreshSelected->setEnabled( oneDirSelected && ! sel->isExcluded() && ! sel->isMountPoint() );
+    bool oneDirSelected	   = selSize == 1 && sel && sel->isDir() && ! sel->isPkgInfo();
+    bool pseudoDirSelected = selectedItems.containsPseudoDir();
+    bool pkgSelected	   = selectedItems.containsPkg();
+
+    _ui->actionMoveToTrash->setEnabled( sel && ! pseudoDirSelected && ! pkgSelected && ! reading );
+    _ui->actionRefreshSelected->setEnabled( oneDirSelected && ! sel->isExcluded() && ! sel->isMountPoint() && ! pkgView );
     _ui->actionContinueReadingAtMountPoint->setEnabled( oneDirSelected && sel->isMountPoint() );
     _ui->actionReadExcludedDirectory->setEnabled      ( oneDirSelected && sel->isExcluded()   );
 
-    bool nothingOrOneDir = selectedItems.isEmpty() ||
-	( selectedItems.size() == 1 && sel && ( sel->isDir() || sel->isDotEntry() ) );
+    bool nothingOrOneDir = selectedItems.isEmpty() || oneDirSelected;
 
     _ui->actionFileSizeStats->setEnabled( ! reading && nothingOrOneDir );
     _ui->actionFileTypeStats->setEnabled( ! reading && nothingOrOneDir );
@@ -541,7 +602,8 @@ void MainWindow::busyDisplay()
     int sortCol = QDirStat::DataColumns::toViewCol( QDirStat::NameCol );
     _ui->dirTreeView->sortByColumn( sortCol, Qt::AscendingOrder );
 
-    if ( ! _selectionModel->currentBranch() )
+    if ( ! PkgFilter::isPkgUrl( _dirTreeModel->tree()->url() ) &&
+	 ! _selectionModel->currentBranch() )
     {
 	// Wait until the toplevel entry has some children, then expand to level 1
 	QTimer::singleShot( 200, _ui->actionExpandTreeLevel1, SLOT( trigger() ) );
@@ -601,30 +663,27 @@ void MainWindow::readingAborted()
 
 void MainWindow::openUrl( const QString & url )
 {
-    QString windowTitle = "QDirStat";
+    if ( PkgFilter::isPkgUrl( url ) )
+	readPkg( url );
+    else if ( isUnpkgUrl( url ) )
+	showUnpkgFiles( url );
+    else
+	openDir( url );
+}
 
-    if ( SysUtil::runningAsRoot() )
-	windowTitle += tr( " [root]" );
 
+void MainWindow::openDir( const QString & url )
+{
     try
     {
 	_dirTreeModel->openUrl( url );
-
-	if ( _urlInWindowTitle )
-	{
-	    // The URL we got at the command line might be relative;
-	    // the DirTree converts this into an absolute URL.
-	    // Let's use that.
-
-	    windowTitle += " " + _dirTreeModel->tree()->url();
-	}
-
-	setWindowTitle( windowTitle );
+	updateWindowTitle( _dirTreeModel->tree()->url() );
     }
     catch ( const SysCallFailedException & ex )
     {
-	setWindowTitle( windowTitle );
 	CAUGHT( ex );
+	updateWindowTitle( "" );
+	_dirTreeModel->tree()->sendFinished();
 
 	QMessageBox errorPopup( QMessageBox::Warning,	// icon
 				tr( "Error" ),		// title
@@ -633,7 +692,7 @@ void MainWindow::openUrl( const QString & url )
 				this );			// parent
 	errorPopup.setDetailedText( ex.what() );
 	errorPopup.exec();
-	askOpenUrl();
+	askOpenDir();
     }
 
     updateActions();
@@ -641,12 +700,132 @@ void MainWindow::openUrl( const QString & url )
 }
 
 
-void MainWindow::askOpenUrl()
+void MainWindow::askOpenDir()
 {
     QString url = QFileDialog::getExistingDirectory( this, // parent
 						     tr("Select directory to scan") );
     if ( ! url.isEmpty() )
+    {
+	_dirTreeModel->tree()->reset();
 	openUrl( url );
+    }
+}
+
+
+void MainWindow::askOpenPkg()
+{
+    bool canceled;
+    PkgFilter pkgFilter = OpenPkgDialog::askPkgFilter( canceled );
+
+    if ( ! canceled )
+    {
+	_dirTreeModel->tree()->reset();
+	readPkg( pkgFilter );
+    }
+}
+
+
+void MainWindow::askShowUnpkgFiles()
+{
+    PkgManager * pkgManager = PkgQuery::primaryPkgManager();
+
+    if ( ! pkgManager )
+    {
+	logError() << "No supported primary package manager" << endl;
+	return;
+    }
+
+    ShowUnpkgFilesDialog dialog( this );
+
+    if ( dialog.exec() == QDialog::Accepted )
+	showUnpkgFiles( dialog.values() );
+}
+
+
+void MainWindow::showUnpkgFiles( const QString & url )
+{
+    UnpkgSettings unpkgSettings( UnpkgSettings::ReadFromConfig );
+    unpkgSettings.startingDir = url;
+
+    showUnpkgFiles( unpkgSettings );
+}
+
+
+void MainWindow::showUnpkgFiles( const UnpkgSettings & unpkgSettings )
+{
+    logDebug() << "Settings:" << endl;
+    unpkgSettings.dump();
+
+    PkgManager * pkgManager = PkgQuery::primaryPkgManager();
+
+    if ( ! pkgManager )
+    {
+	logError() << "No supported primary package manager" << endl;
+	return;
+    }
+
+    _dirTreeModel->clear(); // For instant feedback
+    BusyPopup msg( tr( "Reading file lists..." ), this );
+
+    QString dir = unpkgSettings.startingDir;
+    dir.replace( QRegExp( "^unpkg:" ), "" );
+
+    if ( dir != unpkgSettings.startingDir )
+	logInfo() << "Parsed starting dir: " << dir << endl;
+
+
+    // Set up the exclude rules
+
+    ExcludeRules * excludeRules = new ExcludeRules( unpkgSettings.excludeDirs );
+    CHECK_NEW( excludeRules );
+
+    DirTree * tree = _dirTreeModel->tree();
+    tree->setExcludeRules( excludeRules );
+
+
+    // Prepare the filters with the complete file list of all installed packages
+
+    DirTreeFilter * filter = new DirTreePkgFilter( pkgManager );
+    CHECK_NEW( filter );
+
+    tree->clearFilters();
+    tree->addFilter( filter );
+
+    foreach ( const QString & pattern, unpkgSettings.ignorePatterns )
+    {
+	tree->addFilter( DirTreePatternFilter::create( pattern ) );
+    }
+
+
+    // Start reading the directory
+
+    try
+    {
+	_dirTreeModel->openUrl( dir );
+	updateWindowTitle( _dirTreeModel->tree()->url() );
+    }
+    catch ( const SysCallFailedException & ex )
+    {
+	CAUGHT( ex );
+	updateWindowTitle( "" );
+	_dirTreeModel->tree()->sendFinished();
+
+	QMessageBox errorPopup( QMessageBox::Warning,	// icon
+				tr( "Error" ),		// title
+				tr( "Could not open directory %1" ).arg( ex.resourceName() ), // text
+				QMessageBox::Ok,	// buttons
+				this );			// parent
+	errorPopup.setDetailedText( ex.what() );
+	errorPopup.exec();
+    }
+
+    updateActions();
+}
+
+
+bool MainWindow::isUnpkgUrl( const QString & url )
+{
+    return url.startsWith( "unpkg:/" );
 }
 
 
@@ -657,12 +836,17 @@ void MainWindow::refreshAll()
     if ( ! url.isEmpty() )
     {
 	logDebug() << "Refreshing " << url << endl;
-	_dirTreeModel->openUrl( url );
+
+	if ( PkgFilter::isPkgUrl( url ) )
+	    _dirTreeModel->readPkg( url );
+	else
+	    _dirTreeModel->openUrl( url );
+
 	updateActions();
     }
     else
     {
-	askOpenUrl();
+	askOpenDir();
     }
 }
 
@@ -727,6 +911,30 @@ void MainWindow::askWriteCache()
 }
 
 
+void MainWindow::readPkg( const PkgFilter & pkgFilter )
+{
+    // logInfo() << "URL: " << pkgFilter.url() << endl;
+
+    updateWindowTitle( pkgFilter.url() );
+    expandTreeToLevel( 0 );   // Performance boost: Down from 25 to 6 sec.
+    _dirTreeModel->readPkg( pkgFilter );
+}
+
+
+void MainWindow::updateWindowTitle( const QString & url )
+{
+    QString windowTitle = "QDirStat";
+
+    if ( SysUtil::runningAsRoot() )
+	windowTitle += tr( " [root]" );
+
+    if ( _urlInWindowTitle )
+	windowTitle += " " + url;
+
+    setWindowTitle( windowTitle );
+}
+
+
 void MainWindow::expandTreeToLevel( int level )
 {
     logDebug() << "Expanding tree to level " << level << endl;
@@ -819,16 +1027,16 @@ void MainWindow::notImplemented()
 }
 
 
-void MainWindow::copyCurrentUrlToClipboard()
+void MainWindow::copyCurrentPathToClipboard()
 {
     FileInfo * currentItem = _selectionModel->currentItem();
 
     if ( currentItem )
     {
 	QClipboard * clipboard = QApplication::clipboard();
-	QString url = currentItem->url();
-	clipboard->setText( url );
-	showProgress( tr( "Copied to system clipboard: %1" ).arg( url ) );
+	QString path = currentItem->path();
+	clipboard->setText( path );
+	showProgress( tr( "Copied to system clipboard: %1" ).arg( path ) );
     }
     else
     {
@@ -888,12 +1096,12 @@ void MainWindow::moveToTrash()
 
     foreach ( FileInfo * item, selectedItems )
     {
-	bool success = Trash::trash( item->url() );
+	bool success = Trash::trash( item->path() );
 
 	if ( success )
-	    outputWindow->addStdout( tr( "Moved to trash: %1" ).arg( item->url() ) );
+	    outputWindow->addStdout( tr( "Moved to trash: %1" ).arg( item->path() ) );
 	else
-	    outputWindow->addStderr( tr( "Move to trash failed for %1" ).arg( item->url() ) );
+	    outputWindow->addStderr( tr( "Move to trash failed for %1" ).arg( item->path() ) );
     }
 
     outputWindow->noMoreProcesses();
@@ -919,7 +1127,6 @@ void MainWindow::openConfigDialog()
     _configDialog = new ConfigDialog( this );
     CHECK_PTR( _configDialog );
     _configDialog->cleanupConfigPage()->setCleanupCollection( _cleanupCollection );
-    _configDialog->mimeCategoryConfigPage()->setMimeCategorizer( _mimeCategorizer );
 
     if ( ! _configDialog->isVisible() )
     {
@@ -1083,6 +1290,18 @@ void MainWindow::showHelp()
 }
 
 
+void MainWindow::showPkgViewHelp()
+{
+    SysUtil::openInBrowser( "https://github.com/shundhammer/qdirstat/blob/master/doc/Pkg-View.md" );
+}
+
+
+void MainWindow::showUnpkgViewHelp()
+{
+    SysUtil::openInBrowser( "https://github.com/shundhammer/qdirstat/blob/master/doc/Unpkg-View.md" );
+}
+
+
 void MainWindow::showWhatsNew()
 {
     SysUtil::openInBrowser( RELEASE_URL ); // defined in Version.h
@@ -1099,7 +1318,7 @@ void MainWindow::showAboutDialog()
     text += tr( "Qt-based directory statistics -- showing where all your disk space has gone "
 		" and trying to help you to clean it up." );
     text += "</p><p>";
-    text += "(c) 2015-2018 Stefan Hundhammer";
+    text += "(c) 2015-2019 Stefan Hundhammer";
     text += "</p><p>";
     text += tr( "Contact: " ) + QString( "<a href=\"mailto:%1\">%2</a>" ).arg( mailTo ).arg( mailTo );
     text += "</p><p>";
@@ -1127,8 +1346,8 @@ void MainWindow::showDonateDialog()
     QString text = "<h2>Donate</h2>";
     text += "<p>";
     text += tr( "QDirStat is Free Open Source Software. "
-                "You are not required to pay anything. "
-                "Donations are most welcome, of course." );
+		"You are not required to pay anything. "
+		"Donations are most welcome, of course." );
     text += "</p><p>";
     text += tr( "You can donate any amount of your choice:" );
     text += "</p><p>";
