@@ -18,6 +18,8 @@
 
 #include "ui_main-window.h"
 #include "FileTypeStatsWindow.h"
+#include "PkgFilter.h"
+
 
 class QCloseEvent;
 class QSortFilterProxyModel;
@@ -31,8 +33,8 @@ namespace QDirStat
     class ConfigDialog;
     class DirTreeModel;
     class FileInfo;
-    class MimeCategorizer;
     class SelectionModel;
+    class UnpkgSettings;
 }
 
 using QDirStat::FileInfo;
@@ -56,14 +58,49 @@ public:
 public slots:
 
     /**
-     * Open an URL (start reading that directory).
+     * Open an URL (directory or package URL).
      **/
     void openUrl( const QString & url );
 
+    /**
+     * Open a directory URL (start reading that directory).
+     **/
+    void openDir( const QString & url );
+
     /**
      * Open a directory selection dialog and open the selected URL.
      **/
-    void askOpenUrl();
+    void askOpenDir();
+
+    /**
+     * Open a package selection dialog and open the selected URL.
+     **/
+    void askOpenPkg();
+
+    /**
+     * Open a "show unpackaged files" dialog and start reading the selected
+     * starting dir with the selected exclude dirs.
+     **/
+    void askShowUnpkgFiles();
+
+    /**
+     * Show unpackaged files with the specified 'unpkgSettings' parameters
+     * (startingDir, excludeDirs, ignorePatterns).
+     *
+     * The URL may start with "unpkg:".
+     **/
+    void showUnpkgFiles( const QDirStat::UnpkgSettings & unpkgSettings );
+
+    /**
+     * Show unpackaged files with the UnpkgSettings parameters from the config
+     * file or default values if no config was written yet.
+     **/
+    void showUnpkgFiles( const QString & url );
+
+    /**
+     * Return 'true' if the URL starts with "unpkg:/".
+     **/
+    static bool isUnpkgUrl( const QString & url );
 
     /**
      * Re-read the complete directory tree.
@@ -80,6 +117,12 @@ public slots:
      **/
     void stopReading();
 
+    /**
+     * Clear the current tree and replace it with the list of installed
+     * packages from the system's package manager that match 'pkgUrl'.
+     **/
+    void readPkg( const QDirStat::PkgFilter & pkgFilter );
+
     /**
      * Clear the current tree and replace it with the content of the specified
      * cache file.
@@ -98,6 +141,12 @@ public slots:
      **/
     void askWriteCache();
 
+    /**
+     * Update the window title: Show "[root]" if running as root and add the
+     * URL if that is configured.
+     **/
+    void updateWindowTitle( const QString & url );
+
     /**
      * Expand the directory tree's branches to depth 'level'.
      **/
@@ -124,10 +173,10 @@ public slots:
     void updateFileDetailsView();
 
     /**
-     * Copy the URL of the current item (if there is one) to the system
+     * Copy the path of the current item (if there is one) to the system
      * clipboard for use in other applications.
      **/
-    void copyCurrentUrlToClipboard();
+    void copyCurrentPathToClipboard();
 
     /**
      * Move the selected items to the trash bin.
@@ -149,6 +198,16 @@ public slots:
      **/
     void showHelp();
 
+    /**
+     * Show online help about the packages view.
+     **/
+    void showPkgViewHelp();
+
+    /**
+     * Show online help about the packages view.
+     **/
+    void showUnpkgViewHelp();
+
     /**
      * Show the "about" dialog.
      **/
@@ -363,11 +422,10 @@ private:
     QDirStat::DirTreeModel	* _dirTreeModel;
     QDirStat::SelectionModel	* _selectionModel;
     QDirStat::CleanupCollection * _cleanupCollection;
-    QDirStat::MimeCategorizer	* _mimeCategorizer;
     QDirStat::ConfigDialog	* _configDialog;
     QActionGroup		* _layoutActionGroup;
     QPointer<FileTypeStatsWindow> _fileTypeStatsWindow;
-    QString                       _dUrl;
+    QString			  _dUrl;
     QElapsedTimer		  _stopWatch;
     bool			  _modified;
     bool			  _verboseSelection;
