@@ -17,10 +17,26 @@
 using namespace QDirStat;
 
 
-MimeCategorizer::MimeCategorizer( QObject * parent ):
-    QObject( parent ),
+MimeCategorizer * MimeCategorizer::_instance = 0;
+
+
+MimeCategorizer * MimeCategorizer::instance()
+{
+    if ( ! _instance )
+    {
+	_instance = new MimeCategorizer();
+	CHECK_NEW( _instance );
+    }
+
+    return _instance;
+}
+
+
+MimeCategorizer::MimeCategorizer():
+    QObject( 0 ),
     _mapsDirty( true )
 {
+    // logDebug() << "Creating MimeCategorizer" << endl;
     readSettings();
 }
 
@@ -212,6 +228,7 @@ void MimeCategorizer::readSettings()
 
 void MimeCategorizer::writeSettings()
 {
+    // logDebug() << endl;
     MimeCategorySettings settings;
 
     // Remove all leftover cleanup descriptions
@@ -226,6 +243,7 @@ void MimeCategorizer::writeSettings()
 	MimeCategory * category = _categories.at(i);
 
 	settings.setValue( "Name", category->name() );
+	// logDebug() << "Adding " << groupName << ": " << category->name() << endl;
 	writeColorEntry( settings, "Color", category->color() );
 
 	QStringList patterns = category->humanReadablePatternList( Qt::CaseInsensitive );
@@ -303,6 +321,7 @@ void MimeCategorizer::addDefaultCategories()
 			 << "jpeg"
 			 << "jpg"
 			 << "png"
+			 << "svg"
 			 << "tif"
 			 << "tiff"
 			 << "xcf.bz2"
@@ -336,6 +355,8 @@ void MimeCategorizer::addDefaultCategories()
 			 << "flc"
 			 << "fli"
 			 << "flv"
+			 << "m2ts"
+			 << "m4v"
 			 << "mk3d"
 			 << "mkv"
 			 << "mng"
@@ -363,6 +384,7 @@ void MimeCategorizer::addDefaultCategories()
 			<< "aac"
 			<< "f4a"
 			<< "f4b"
+			<< "flac"
 			<< "m4a"
 			<< "m4b"
 			<< "mid"
@@ -432,7 +454,7 @@ void MimeCategorizer::addDefaultCategories()
     add( src );
     src->addSuffixes( QStringList()
 		      << "c" << "cpp" << "cc" << "cxx" << "h" << "hpp" << "ui"
-		      << "pl" << "py" << "rb" << "el"
+		      << "pl" << "py" << "rb" << "el" << "js" << "php" << "java"
 		      << "pro" << "cmake"
 		      , Qt::CaseSensitive );
 
@@ -442,7 +464,7 @@ void MimeCategorizer::addDefaultCategories()
     add( obj );
 
     obj->addSuffixes( QStringList()
-		      << "o" << "lo" << "Po" << "al" << "la" << "moc" << "elc" << "log"
+		      << "o" << "lo" << "ko" << "Po" << "al" << "la" << "moc" << "elc" << "log" << "pyc"
 		      , Qt::CaseSensitive );
 
     obj->addPatterns( QStringList()
@@ -458,5 +480,5 @@ void MimeCategorizer::addDefaultCategories()
     libs->addPattern( "lib*.so",   Qt::CaseSensitive );
     libs->addPattern( "lib*.a",	   Qt::CaseSensitive );
     libs->addSuffix ( "dll" );
-
+    libs->addSuffix ( "so" );
 }
